# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""Interface to the Reactome webs services

.. topic:: What is Reactome?

    :URL: http://www.reactome.org/ReactomeGWT/entrypoint.html
    :Citation: http://www.reactome.org/citation.html
    :REST: http://reactomews.oicr.on.ca:8080/ReactomeRESTfulAPI/RESTfulWS

    .. highlights::

        "REACTOME is an open-source, open access, manually curated and peer-reviewed
        pathway database. Pathway annotations are authored by expert biologists, in
        collaboration with Reactome editorial staff and cross-referenced to many
        bioinformatics databases. These include NCBI Entrez Gene, Ensembl and UniProt
        databases, the UCSC and HapMap Genome Browsers, the KEGG Compound and ChEBI
        small molecule databases, PubMed, and Gene Ontology. ... "

        -- from Reactome web site

"""
from bioservices.services import WSDLService, REST
import webbrowser
import copy

__all__ = ['Reactome', 'ReactomeAnalysis']
# for reactome, content-type could be
#  "Content-Type", "multipart/form-data; boundary=" +    boundary);


class Reactome(REST):
    """Reactome interface

    some data can be download on the main `website <http://www.reactome.org/pages/download-data/>`_
    """

    _url = "http://reactomews.oicr.on.ca:8080/ReactomeRESTfulAPI/RESTfulWS"

    def __init__(self, verbose=True, cache=False):
        super(Reactome, self).__init__("Reactome(URL)",url=Reactome._url,
            verbose="ERROR", cache=False)
        self.debugLevel = verbose
        self.test = 2

        # for buffering
        self._list_pathways = None

    def _download_list_pathways(self):
        if self._list_pathways is None:
            res =  self.session.get("http://www.reactome.org/download/current/ReactomePathways.txt")
            if res.status_code == 200:
                res = res.text # content does not work in python 3.3
                res = res.strip()
                self._list_pathways = [x.split("\t") for x in res.split("\n")]
            else:
                self.logging.error("could not fetch the pathways")
        return self._list_pathways

    def get_list_pathways(self):
        """Return list of pathways from reactome website

        :return: list of lists. Each sub-lis contains 3 items: reactome pathway
            identifier, description and species

        """
        res = self._download_list_pathways()
        return res

    def get_species(self):
        """Return list of species from all pathways"""
        res = self._download_list_pathways()
        res = set([x[2] for x in self.get_list_pathways()])
        return res


    def biopax_exporter(self, identifier, level=2):
        """Get BioPAX file

        The passed identifier has to be a valid event identifier. If there is no matching ID in
        the database, it will return an empty string.

        :param int level: BioPAX level: one of two values: 2 or 3
        :param int identfier: event database identifier
        :return: BioPAX RDF document


        ::

            >>> # for Apoptosis:
            >>> s = Reactome()
            >>> res = s.biopax_exporter(109581)
        """
        res = self.http_get("biopaxExporter/Level{0}/{1}".format(level, identifier),
                frmt=None)
        return res

    def front_page_items(self, species):
        """Get list of front page items listed in the Reactome Pathway Browser

        :param str species: Full species name that should be encoded for URL (e.g.
            homo+sapiens for human, or mus+musculus for mouse) + can be replaced
            by spaces.

        :return:  list of fully encoded Pathway objects in JSON

        ::

            >>> s = Reactome()
            >>> res = s.front_page_items("homo sapiens")
            >>> print(res[0]['name'])
            ['Apoptosis']

        .. seealso:: `Pathway Browser <http://www.reactome.org/PathwayBrowser/>`_
        """
        species = species.replace("+", " ")
        res = self.http_get("frontPageItems/{0}".format(species),
                frmt="json")
        return res

    def highlight_pathway_diagram(self, identifier, genes, frmt="PNG"):
        """Highlight a diagram for a specified pathway based on its identifier

        :param int identifier: a valid pathway  identifier
        :param list genes: a list of string to indicate the genes to highlight
        :param int frmt: PNG or PDF
        :return:  This  method should be used after method queryHitPathways.

        ::

            res = s.http_post("highlightPathwayDiagram/68875/PNG", frmt="txt",
                data="CDC2")
            with open("test.png", 'wb') as f:
                f.write(res.decode("base64"))
                f.close()

        .. todo:: Saving the image above returns a blank image ...


        """
        self.devtools.check_param_in_list(frmt, ['PDF', 'PNG'])

        url = "highlightPathwayDiagram/{0}/{1}"
        genes = self.devtools.list2string(genes)

        res = self.http_post(url.format(identifier, frmt), frmt="txt",
                data=genes)
        return res


    def list_by_query(self, classname, **kargs):
        """Get list of objecs from Reactome database

        :param str class name:
        :param kargs: further attribute values encoded in key-value pair
        :return: list of dictionaries. Each dictionary contains information
            about a given pathway

        To query a list of pathways with names as "Apoptosis"::

            >>> s = Reactome()
            >>> res = list_by_query("Pathway", name="apoptosis")
            >>> identifiers = [x['dbId'] for x in res]

        """
        url = "listByQuery/{0}".format(classname)
        # NOTE: without the content-type this request fails with error 415
        # fixed by
        res = self.http_post(url, frmt='json', data=kargs,
                headers={'Content-Type': "application/json;odata=verbose"})
        return res

    def pathway_diagram(self, identifier, frmt="PNG"):
        """Retrieve pathway diagram

        :param int identifier: Pathway database identifier
        :param str frmt: PNG, PDF, or XML.
        :return:  Base64 encoded pathway diagram for PNG or PDF. XML text for the XML file type.

        ::

            >>> s = Reactome()
            >>> s.pathway_diagram('109581', 'PNG',view=True)
            >>> s.pathway_diagram('109581', 'PNG', save=True)

        .. todo:: if PNG or PDF, the output is base64 but there is no
            facility to easily save the results in a file for now
        """
        self.devtools.check_param_in_list(frmt, ['PDF', 'PNG', 'XML'])
        url = 'pathwayDiagram/{0}/{1}'.format(identifier, frmt)
        res = self.http_get(url, frmt=frmt)
        return res

    def pathway_hierarchy(self, species):
        """Get the pathway hierarchy for a species as displayed in  Reactome pathway browser.

        :param str species: species name that should be with + or spaces (e.g.
            'homo+sapiens' for  human, or 'mus musculus' for mouse)
        :return: XML text containing  pathways and reactions

        ::

            s.pathway_hierarchy("homo sapiens")
        """
        species = species.replace("+", " ")
        res = self.http_get("pathwayHierarchy/{0}".format(species),
                            frmt="xml")
        return res

    def pathway_participants(self, identifier):
        """Get list of pathway participants for a pathway using

        :param int identifier: Pathway database identifier
        :return: list of fully encoded PhysicalEntity objects in the pathway
            (in JSON)

        ::

            >>> s = Reactome()
            >>> s.pathway_participants(109581)
        """
        res = self.http_get("pathwayParticipants/{0}".format(identifier),
                            frmt='json')
        return res

    def pathway_complexes(self, identifier):
        """Get complexes belonging to a pathway

        :param int identifier: Pathway database identifier
        :return: list of all PhysicalEntity objects that participate in the
            Pathway.(in JSON)

        ::

            >>> s = Reactome()
            >>> s.pathway_complexes(109581)

        """
        res = self.http_get("pathwayComplexes/{0}".format(identifier),
                frmt="json")
        return res

    def query_by_id(self, classname, identifier):
        """Get Reactome Database for a specific object.
        

        :param str classname: e.g. Pathway
        :param int identifier: database identifier or stable identifier if available

        It returns a full object, including full class information about
        all the attributes of the returned object. For example, if the object has
        one PublicationSource attribute, it will return a full PublicationSource
        object within the returned object.

        ::

            >>> s.query_by_id("Pathway", "109581")

        """
        url = "queryById/{0}/{1}".format(classname, identifier)
        res = self.http_get(url, frmt='json')
        return res

    def query_by_ids(self, classname, identifiers):
        """

        :param str classname: e.g. Pathway
        :param list identifiers: list of strings or int


        ::

            >>> s.quey_by_ids("Pathway", "CDC2")

        .. warning:: not sure the wrapping is correct
        """

        identifiers = self.devtools.list2string(identifiers)
        url = "queryByIds/{0}".format(classname)
        res = self.http_post(url, frmt="json", data=identifiers)
        #headers={'Content-Type': "application/json"})
        return res

    def query_hit_pathways(self, query):
        """Get pathways that contain one or more genes passed in the query list.

        In the Reactome data model, pathways are organized in a
        hierarchical structure. The returned pathways in this method are pathways
        having detailed manually drawn pathway diagrams. Currently only human
        pathways will be returned from this method.

        ::

            s.query_hit_pathways('CDC2')
            s.query_hit_pathways(['CDC2'])

        """
        identifiers = self.devtools.list2string(query)
        res = self.http_post("queryHitPathways", frmt='json', data=identifiers)
        return res

    def query_pathway_for_entities(self, identifiers):
        """Get pathway objects by specifying an array of PhysicalEntity database identifiers.


        The returned Pathways should
        contain the passed EventEntity objects. All passed EventEntity database
        identifiers should be in the database.

        """
        identifiers = self.devtools.list2string(identifiers, space=False)
        url = "pathwayForEntities"
        res = self.http_post(url, frmt='json', data={'ID':identifiers})
        return res

    def species_list(self):
        """Get the list of species used Reactome"""
        url = "speciesList"
        res = self.http_get(url, frmt='json')
        return res

    def SBML_exporter(self, identifier):
        """Get the SBML XML text of a pathway identifier

        :param int identifier: Pathway database identifier
        :return: SBML object in XML format as a string


        ::

            >>> from bioservices import Reactome
            >>> s = Reactome()
            >>> xml = s.SBML_exporter(109581)

        """
        url = "sbmlExporter/{0}".format(identifier)
        res = self.http_get(url, frmt='xml')
        return res


    def get_all_reactions(self):
        """Return list of reactions from the Pathway"""
        res = self.get_list_pathways()
        return [x[0] for x in res]

    def bioservices_get_reactants_from_reaction_identifier(self, reaction):
        """Fetch information from the reaction HTML page

        .. note:: draft version
        """
        res = self.http_get('http://www.reactome.org/content/detail/%s' % reaction) 
        res = res.content

        try:
            reactants = [x for x in res.split("\n") if '<title>' in x]
            reactants = reactants[0].split("|")[1].strip().strip('</title>')
        except  Exception as err:
            print('Could not interpret title')
            return res

        if reactants.count(':') == 1:
            reactants = reactants.split(":")
        else:
            #self.logging.warning('Warning: did not find unique sign : for %s' % reaction)
            #reactants = reactants.split(":", 1)
            pass

        return reactants


"""
class ReactomePathway(object):

    def __init__(self, entry):
        self.raw_data = copy.deepcopy(entry)
        # just adding the attributes to make life easier
        for k in self.raw_data._keyord:
            setattr(self, k, getattr(self.raw_data, k))

    def __str__(self):

        txt = "id: " + str(self.id)
        txt += "\nname: " + str(self.name)

        txt += "\nhasComponent:"
        if self.hasComponent:
            txt += str(len(self.hasComponent))

        if self.raw_data.compartment:
            txt += "\ncompartment:" + str(len(self.raw_data.compartment))
        else:
            txt += "\ncompartment: " + str(self.raw_data.compartment)

        txt += "\nliteratureReference:"
        this = self.raw_data.literatureReference
        if this:
            for i,x in enumerate(this):
                txt += " --- %s: %s %s\n" % (i, x.id, x.name)

        txt += "\nspecies:"
        this = self.raw_data.species
        if this:
            txt += "  %s (%s)" % (this.scientificName, this.id)

        txt += "\nsummation:"
        this = self.raw_data.summation
        if this:
            for x in this:
                txt += " - %s \n" % (self.raw_data.id)

        txt += "\ngoBiologicalProcess:"
        this = self.raw_data.goBiologicalProcess
        if this: txt += "\n - %s %s\n" % (this.id, this.name)

        txt += "\ninferredFrom:" + str(self.raw_data.inferredFrom)
        txt += "\northologousEvent: "
        this = self.raw_data.orthologousEvent
        if this:
            txt += str(len(this))

        txt += "\nprecedingEvent: " + str(self.raw_data.precedingEvent)
        txt += "\nevidence Type: " + str(self.raw_data.evidenceType) + "\n"

        return txt
"""


class ReactomeAnalysis(REST):


    _url = "http://www.reactome.org:80/AnalysisService"
    
    #"identifiers/projection?pageSize=8000&page=1&sortBy=ENTITIES_PVALUE&order=ASC&resource=TOTAL",

    def __init__(self, verbose=True, cache=False):
        super(ReactomeAnalysis, self).__init__("Reactome(URL)",url=ReactomeAnalysis._url,
            verbose=verbose, cache=False)
        self.logging.warning("Class in development. Some methods are already  working but those required POST do not. Coming soon ")



    def identifiers(self, genes):
        """

        s.identfiers("TP53")
        .. warning:: works for oe gene only for now
        """
        url = "identifiers/projection?pageSize=8000&page=1&sortBy=ENTITIES_PVALUE&order=ASC&resource=TOTAL"

        genes = self.devtools.list2string(genes)
        genes = genes.replace(" ","")
        print(genes)
        res = self.http_post(url, frmt="json", data=genes,
                headers={"Content-Type": "text/plain;charset=UTF-8", 
                    "Accept": "application/json"})
        return res








#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$

"""This module provides a class :class:`IntactComplex`

.. topic:: What is Intact Complex ?

    :URL:  https://www.ebi.ac.uk/intact/complex/
    :REST:  https://www.ebi.ac.uk/intact/complex-ws/details/


    .. highlights::

        "The Complex Portal is a manually curated, encyclopaedic resource of
        macromolecular complexes from a number of key model organisms."

        -- From Intact web page Feb 2015


"""
from bioservices import REST


__all__ = ['IntactComplex']


class Intact(object):
    def __init__(self):
        print("Not implemented yet. For Intact Complex, please use IntactComplex class")


class IntactComplex(REST):
    """Interface to the `Intact <http://www.ebi.ac.uk/intact/>`_ service

    .. doctest::

            >>> from bioservices import IntactComplex
            >>> u = IntactComplex()

    """

    _url = "http://www.ebi.ac.uk/intact/complex-ws"

    def __init__(self, verbose=False, cache=False):
        """**Constructor** IntactComplex

        :param verbose: set to False to prevent informative messages
        """
        super(IntactComplex, self).__init__(name="IntactComplex", url=IntactComplex._url,
                verbose=verbose, cache=cache)

    def search(self, query, frmt='json', facets=None, first=None, number=None, filters=None):
        """Search for a complex inside intact complex.

        :param str query: the query (e.g., ndc80)
        :param str frmt: Defaults to json (could be a Pandas data frame if
            Pandas is installed; set frmt to 'pandas')
        :param str facets: lists of facets as a string (separated by comma)
        :param int first:
        :param int number:
        :param str filter: list of filters. See examples here below.

        .. code-block:: python

            s = IntactComplex()
            # search for ndc80
            s.search('ncd80')

            #  Search for ndc80 and facet with the species field:
            s.search('ncd80', facets='species_f')

            # Search for ndc80 and facet with the species and biological role fields:
            s.search('ndc80', facets='species_f,pbiorole_f')

            # Search for ndc80, facet with the species and biological role
            # fields and filter the species using human:
            s.search('Ndc80', first=0, number=10,
                filters='species_f:("Homo sapiens")',
                facets='species_f,ptype_f,pbiorole_f')

            # Search for ndc80, facet with the species and biological role
            # fields and filter the species using human or mouse:
            s.search('Ndc80, first=0, number=10,
                filters='species_f:("Homo sapiens" "Mus musculus")',
                facets='species_f,ptype_f,pbiorole_f')

            # Search with a wildcard to retrieve all the information:
            s.search('*')

            # Search with a wildcard to retrieve all the information and facet
            # with the species, biological role and interactor type fields:
            s.search('*', facets='species_f,pbiorole_f,ptype_f')

            # Search with a wildcard to retrieve all the information, facet with
            # the species, biological role and interactor type fields and filter
            # the interactor type using small molecule:
            s.search('*', facets='species_f,pbiorole_f,ptype_f',
                filters='ptype_f:("small molecule")'

            # Search with a wildcard to retrieve all the information, facet with
            # the species, biological role and interactor type fields and filter
            # the interactor type using small molecule and the species using human:
            s.search('*', facets='species_f,pbiorole_f,ptype_f',
                filters='ptype_f:("small molecule"),species_f:("Homo sapiens")')

            # Search for GO:0016491 and paginate (first is for the offset and number
            # is how many do you want):
            s.search('GO:0016491', first=10, number=10)

        The organism name used in the filter must be exact. Here is the list
        found by typing::

            res = set(ci.search('*', frmt='pandas')['organismName'])

        ::

           'Bos taurus; 9913',
           'Caenorhabditis elegans; 6239',
           'Canis familiaris; 9615',
           'Drosophila melanogaster; 7227',
           'Escherichia coli (strain K12); 83333',
           'Gallus gallus; 9031',
           'Homo sapiens; 9606',
           'Mus musculus; 10090',
           'Oryctolagus cuniculus; 9986',
           'Rattus norvegicus; 10116',
           'Saccharomyces cerevisiae (strain ATCC 204508 / S288c);559292',
           'Schizosaccharomyces pombe (strain 972 / ATCC 24843);284812',
           'Xenopus laevis; 8355'


        """
        self.devtools.check_param_in_list(frmt, ['pandas', 'json'])

        # note that code format to be json, which is the only option so
        # we can use pandas as a frmt without addition code.
        params = {'format': 'json', 'facets':facets, 'first':None,
                'number':number, 'filters':filters}

        result = self.http_get('search/' + query, frmt="json", params=params)

        #if isinstance(result, int):
        #    raise ValueError("Got a number from Intact request. Check validity of the arguments ")

        if frmt == 'pandas':
            import pandas as pd
            df = pd.DataFrame(result['elements'])
            return df
        else:
            return result

    def details(self, query):
        """Return details about a complex

        :param str query: EBI-1163476

        """
        result = self.http_get('details/' + query, frmt="json")
        return result





#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""ChEMBLdb has been renamed into ChEMBL (module chembl instead of chembldb)"""


class ChEMBLdb(object):
    def __init__(self):
        print("""This class is deprecated. To update your code, 
            1. replace the module chembldb by chembl
            2. replace the class ChEMBLdb by ChEMBL""")







# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id$
"""Interface to the quickGO interface

.. topic:: What is quickGO

    :URL: http://www.ebi.ac.uk/QuickGO/
    :Service: http://www.ebi.ac.uk/QuickGO/WebServices.html

    .. highlights::

        "QuickGO is a fast web-based browser for Gene Ontology terms and
        annotations, which is provided by the UniProt-GOA project at the EBI. "

        -- from QuickGO home page, Dec 2012

"""
from __future__ import print_function

from bioservices.services import REST

__all__ = ["QuickGO"]


class QuickGO(REST):
    """Interface to the `QuickGO <http://www.ebi.ac.uk/QuickGO/WebServices.html>`_ service

    Retrieve information given a GO identifier:

    .. doctest::

        >>> from bioservices import QuickGO
        >>> s = QuickGO()
        >>> res = s.Term("GO:0003824")

    Retrieve information about a protein given its uniprot identifier, a
    taxonomy number. Let us also restrict the search to the UniProt database and
    print only 3 columns of information (protein name, GO identifier and GO
    name)::

        print(s.Annotation(protein="Q8IYB3", frmt="tsv", tax=9606,
            source="UniProt", col="proteinName,goID,goName"))

    Here is the Term output for a given GO identifier::

        >>> print(s.Term("GO:0000016", frmt="obo"))
        [Term]
        id: GO:0000016
        name: lactase activity
        def: "Catalysis of the reaction: lactose + H2O = D-glucose + D-galactose."
        synonym: "lactase-phlorizin hydrolase activity" broad
        synonym: "lactose galactohydrolase activity" exact
        xref: EC:3.2.1.108
        xref: MetaCyc:LACTASE-RXN
        xref: RHEA:10079
        is_a: GO:0004553 ! hydrolase activity, hydrolyzing O-glycosyl compounds


    """
    _goid_example = "GO:0003824"
    _valid_col = ['proteinDB', 'proteinID', 'proteinSymbol', 'qualifier',
            'goID', 'goName', 'aspect', 'evidence', 'ref', 'with', 'proteinTaxon',
            'date', 'from', 'splice', 'proteinName', 'proteinSynonym', 'proteinType',
            'proteinTaxonName', 'originalTermID', 'originalGOName']

    def __init__(self, verbose=False, cache=False):
        """.. rubric:: Constructor

        :param bool verbose: print informative messages.

        """
        super(QuickGO, self).__init__(url="http://www.ebi.ac.uk/QuickGO",
            name="quickGO", verbose=verbose, cache=cache)

    def Term(self, goid, frmt="oboxml"):
        """Obtain Term information


        :param str frmt: the output format (mini, obo, oboxml).

        The format can be:

        * mini:   Mini HTML, suitable for dynamically embedding in popup boxes.
        * obo:    OBO format snippet.
        * oboxml: OBO XML format snippet.

        ::

            from bioservices import QuickGO
            s = QuickGO()
            s.Term("GO:0003824")


        """
        self.devtools.check_param_in_list(frmt, ["mini", "obo", "oboxml"])
        if goid.startswith("GO:")is False:
            raise ValueError("GO id must start with 'GO:'")

        params = {'id':goid, 'format': frmt}
        res = self.http_get("GTerm", frmt="xml", params=params)

        return res

    def Annotation(self, goid=None, protein=None, frmt="gaf", limit=10000,
            gz=False, col=None, db=None, aspect=None, relType=None, termUse=None,
            evidence=None, source=None, ref=None,  tax=None, qualifier=None, q=None, 
            _with=None):
        """Calling the Annotation service

        Mutual exclusive parameters are goid, protein

        :param int limit: download limit (number of lines) (default 10,000 rows,
            which may not be sufficient for the data set that you are
            downloading. To bypass this default, and return the entire data set,
            specify a limit of -1).
        :param str frmt: one of "gaf", "gene2go", "proteinList", "fasta",
            "tsv" or "dict". The "dict" argument is the default and is a
            python dictionary.
        :param bool gz: gzips the downloaded file.
        :param str goid: GO identifiers either directly or indirectly
            (descendant GO identifiers) applied in annotations.
        :param char aspect: use this to limit the annotations returned to a
            specific ontology or ontologies (Molecular Function, Biological
            Process or Cellular Component). The valid character can be F,P,C.
        :param relType: not Implemented. By default, QuickGO will display annotations to GO terms
            that are related to that specified in the goid parameter by is_a,
            part_of and occurs_in relations; this parameter allows you to
            override that behaviour. See `details <http://www.ebi.ac.uk/QuickGO/reference.html#slim_custom>`_
        :param termUse:  if you set this parameter to slim, then QuickGO will
            use the supplied set of GO identifiers as a slim and will map the
            annotations up to these terms. See here for more details:
            http://www.ebi.ac.uk/QuickGO/GMultiTerm
        :param str db: protein database (identifier type). Can be UniProtKB, UniGene, Ensembl.
        :param str evidence: annotation evidence code category (Ev). Example of
            valid evidence are: be IDA, IC, ISS, IEA, IPI, ND, IMP, ISO, IGI
            should be either a string with comma separated values (e.g.,
            IEA,IDA) or a list of strings (e.g. ["IEA","IDA"]).
        :param str source: annotation provider. Examples are 'InterPro', 'UniPathway',
            'MGI', 'FlyBase', 'GOC', 'Source', 'UniProtKB', 'RGD', 'ENSEMBL',
            'ZFIN', 'IntAct'.
        :param str ref: PubMed or GO reference supporting annotation. Can refer to a
            specific reference identifier or category (for category level, use
            `*`  after ref type). Can be 'PUBMED:`*`', 'GO_REF:0000002'.
        :param str with: additional supporting information supplied in IEA, ISS, IPI, IC
            evidenced annotations; see GO documentation. Can refer to a specific
            identifier or category (for category level, use * after with type).
            Examples are: EC:2.5.1.30, IPR000092, HAMAP:*
        :param str qualifier: tags that modify the interpretation of an annotation.
             Examples are NOT, colocalizes_with, contributes_to.

        .. note::
            * Any number of fields can be specified; they will be AND'ed together.
            * Any number of values can be specified for each field; they will be OR'ed together.
            * Values should be URI encoded.
            * The file will be truncated if more than the specified number of annotations are found. The file is roughly 170 bytes/annotation (not gzipped).
            * The file will be gzipped if the gz parameter is supplied.


        ::

            >>> print(s.Annotation(protein='P12345', frmt='tsv', col="ref,evidence",
            ... ref='PMID:*'))
            >>> print(s.Annotation(protein='P12345,Q4VCS5', frmt='tsv',
            ...     col="ref,evidence",ref='PMID:,Reactome:'))



        """
        _valid_formats = ["gaf", "gene2go", "proteinList", "fasta", "tsv", "dict"]
        _valid_db = ['UniProtKB', 'UniGene', 'Ensembl']
        _valid_aspect = ['P', 'F', 'C']

        self.devtools.check_param_in_list(frmt, _valid_formats)

        if isinstance(limit, int) is False:
            raise TypeError("limit parameter must be an integer greater than zero")

        # fill params with parameters that have default values.
        params = {'format': frmt, 'limit': limit}

        # beginning of the URL
        url = "GAnnotation?"

        # what is the ID being provided. We can have only one of:
        # protein, goid
        if protein is not None:
            url += "protein=" + protein
        elif goid is not None:
            url += "goid=" + goid
        elif tax is not None:
            url += "tax=" + tax

        #need to check that there are mutualy exclusive
        if goid is None and protein is None and tax is None:
            raise ValueError("you must provide at least one of the following parameter: goid, protein")

        # aspect parameter
        if aspect is not None:
            self.devtools.check_param_in_list(aspect, _valid_aspect)
            params['aspect'] = aspect

        # aspect parameter
        if termUse is not None:
            self.devtools.check_param_in_list(termUse, ["slim"])
            params['termUse'] = termUse

        if relType:
            raise NotImplementedError

        if q:
            raise NotImplementedError

        if evidence:
            if isinstance(evidence,list):
                evidence = ",".join([x.strip() for x in evidence])
            elif isinstance(evidence,str):
                pass
            else:
                raise ValueError("""
Invalid parameter: evidence parameters must be a list of strings ['IDA','IEA']
or a string (e.g., 'IDA', 'IDA,IEA') """)
            params['evidence'] = evidence

        if source:
            if isinstance(source, list):
                source = ",".join([x.strip() for x in source])
            elif isinstance(source, str):
                pass
            else:
                raise ValueError("""
Invalid parameter: source parameters must be a list of strings ['UniProtKB']
or a string (e.g., 'UniProtKB') """)
            params['source'] = source

        if ref:
            if isinstance(ref, list):
                ref = ",".join([x.strip() for x in ref])
            elif isinstance(ref, str):
                pass
            else:
                raise ValueError("""
Invalid parameter: source parameters must be a list of strings ['PUBMED']
or a string (e.g., 'PUBMED:*') """)
            params['ref'] = ref

        if qualifier:
            #NOT, colocalizes_with, contributes_to
            if isinstance(qualifier, list):
                qualifier = ",".join([x.strip() for x in qualifier])
            elif isinstance(qualifier, str):
                pass
            params['qualifier'] = qualifier

        # col parameter
        if frmt == "tsv":
            if col is None:
                col = 'proteinDB,proteinID,proteinSymbol,qualifier,'
                col += 'goID,goName,aspect,evidence,ref,with,proteinTaxon,'
                col += 'date,from,splice,proteinName,proteinSynonym,proteinType,'
                col += 'proteinTaxonName,originalTermID,originalGOName'
            else:
                col = ",".join([x.strip() for x in col.split(",")])

            for c in col.split(','):
                self.devtools.check_param_in_list(c, self._valid_col)
            params["col"] = col

        if frmt not in ["tsv", "dict"]:
            # col is provided but format is not appropriate
            if col is not None:
                raise ValueError("You provided the 'col' parameter but the format is not correct. You should use the frmt='tsv' or frmt='dict' ")

        # gz parameter. do not expect values so need to be added afterwards.
        if gz is True:
            url += '&gz'

        res = self.http_get(url, frmt="txt", params=params)

        return res

    def Annotation_from_goid(self, goid, **kargs):
        """Returns a DataFrame containing annotation on a given GO identifier

        :param str protein: a GO identifier
        :return: all outputs are stored into a Pandas.DataFrame data structure.

        All parameters from :math:`Annotation` are also valid except **format** that
        is set to **tsv**  and cols that is made of all possible column names.

        """
        kargs["frmt"] = "tsv"
        cols = ",".join (self._valid_col)
        kargs['col'] = cols

        data = self.Annotation(goid=goid, **kargs)
        data = data.strip().split("\n")[1:]
        res = {}
        for c in cols.split(","):
            res[c] = []

        for entry in data:
            values = entry.split("\t")
            for k, v in zip(cols.split(","), values):
                res[k].append(v)
        try:
            import pandas as pd
            return pd.DataFrame(res)
        except:
            self.logging.warning("Cannot return a DataFrame. Returns the list. If you want the dataframe, install pandas library")
            return res

    def Annotation_from_protein(self, protein, **kargs):
        """Returns a DataFrame containing annotation on a given protein

        :param str protein: a protein name
        :return: all outputs are stored into a Pandas.DataFrame data structure.

        All parameters from :math:`Annotation` are also valid except **format** that
        is set to **tsv**  and cols that is made of all possible column names.

        """
        kargs["frmt"] = "tsv"
        cols = ",".join (self._valid_col)
        kargs['col'] = cols

        data = self.Annotation(protein=protein, **kargs)
        data = data.strip().split("\n")[1:]
        res = {}
        for c in cols.split(","):
            res[c] = []

        for entry in data:
            values = entry.split("\t")
            for k, v in zip(cols.split(","), values):
                res[k].append(v)
        try:
            import pandas as pd
            return pd.DataFrame(res)
        except:
            self.logging.warning("Cannot return a DAtaFrame. Returns the list")
            return res



class GeneOntology():
    """


    Given a list of GO terms, read them with QuickGO and convert them to a database.

    [Term]
    id: GO:0031655
    name: negative regulation of heat dissipation
    def: "Any process that stops, prevents, or reduces the rate or extent of heat dissipation."
    synonym: "downregulation of heat dissipation" exact
    synonym: "inhibition of heat dissipation" narrow
    synonym: "down-regulation of heat dissipation" exact
    synonym: "down regulation of heat dissipation" exact
    is_a: GO:0031654 ! regulation of heat dissipation
    is_a: GO:0032845 ! negative regulation of homeostatic process
    is_a: GO:0051241 ! negative regulation of multicellular organismal process


    """
    def __init__(self):
        self._quickgo = QuickGO(verbose=False)

    def getGOTerm(self, goid):
        return self._quickgo.Term(goid, frmt="obo")





from bioservices import BioMart
from nose.plugins.attrib import attr


class test_biomart(object):
    @classmethod
    def setup_class(klass):
        # ideally we should not provide a host to be more generic 
        # but this takes lots of time or is simply down.
        klass.s = BioMart(host='www.ensembl.org', verbose=False)
        klass.mart_test = 'ENSEMBL_MART_ENSEMBL'

    def test_general(self):
        # test another host
        s = BioMart(host="www.ensembl.org")

    def test_version(self):
        self.s.version(self.mart_test)

    def test_datasets(self):
        # there are about 70 dataset but let us check that at least the list is
        # not empty
        assert len(self.s.datasets(self.mart_test)) > 2

        assert "mmusculus_gene_ensembl" in self.s.datasets(self.mart_test)

    def test_attributes(self):
        assert 'oanatinus_gene_ensembl' in \
            self.s.valid_attributes[self.mart_test]

    def test_filteres(self):
        self.s.filters("oanatinus_gene_ensembl")

    def test_config(self):
        self.s.configuration("oanatinus_gene_ensembl")

    def test_query(self):
        res = self.s.query(self.s._xml_example)
        assert "ENSMUS" in res

    def test_xml(self):
        # build own xml using the proper functions
        self.s.add_dataset_to_xml("mmusculus_gene_ensembl")
        self.s.get_xml()

def _test_reactome_example():
    # this is not working anymore...
    s = BioMart()
    s.lookfor("reactome")
    s.datasets("REACTOME")
    #['interaction', 'complex', 'reaction', 'pathway']
    s.new_query()
    s.add_dataset_to_xml("pathway")
    s.add_filter_to_xml("species_selection", "Homo sapiens")
    s.add_attribute_to_xml("pathway_db_id")
    s.add_attribute_to_xml("_displayname")
    xmlq = s.get_xml()
    res = s.query(xmlq)




from bioservices import picr
from nose.plugins.attrib import attr

# timeout on travis...
@attr('skip_travis')
class TestPICR(object):

    @classmethod
    def setup_class(klass):
        klass.e = picr.PICR(verbose=False)

    def test_getUPIForSequence(self):
        res = self.e.getUPIForSequence(self.e._sequence_example,
                ["IPI", "ENSEMBL", "SWISSPROT"])
        res = self.e.getUPIForSequence(self.e._sequence_example, "SWISSPROT",
                taxid="9606")
        res = self.e.getUPIForSequence(self.e._sequence_example, "SWISSPROT",
                onlyactive=False, includeattributes=False)

    def test_databases(self):
        assert len(self.e.databases) > 0

    @attr('slow')
    def test_MappedDB(self):
        self.e.getMappedDatabaseNames()

    @attr('slow')
    def test_checkDB(self):
        self.e._checkDBname("IPI")
        try:
            self.e._checkDBname("dummy")
            assert False
        except:
            assert True

    @attr('slow')
    def test_getUPIForAccession(self):
        self.e.getUPIForAccession(self.e._accession_example, ["SWISSPROT"])
        self.e.getUPIForAccession(self.e._accession_example, "SWISSPROT", taxid="9606")
        res = self.e.getUPIForAccession(self.e._accession_example, "SWISSPROT", onlyactive=False, includeattributes=False)

    # this one is failing from time to time even the exemple on the web site.
    @attr('fixme')
    def test_getUPIForBLAST(self):
        self.e.getUPIForBLAST(self.e._blastfrag_example, "SWISSPROT", taxid="9606")
        self.e.getUPIForBLAST(self.e._blastfrag_example, ["SWISSPROT"], taxid="9606", includeattributes=False)
        self.e.getUPIForBLAST(self.e._blastfrag_example, ["SWISSPROT"], taxid="9606", program="blastp",matrix="BLOSUM62")



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""This module provides a class :class:`~Readseq` to access to Readseq WS.


.. topic:: What is Readseq ?

    :URL: http://www.ebi.ac.uk/Tools/sfc/readseq/
    :Service: 
    :Citations: http://www.ncbi.nlm.nih.gov/pubmed/18428689

    .. highlights::

        Readseq reads and converts biosequences between a selection of common 
        biological sequence formats, including EMBL, GenBank and fasta sequence 
        formats. 

        Readseq homepage -- Sep 2014



.. testsetup:: biomodels

    from bioservices import Readseq
    s = Readseq()


"""
import copy
from bioservices.services import WSDLService


__all__ = ["Readseq"]



class Readseq(WSDLService):
    """Interface to the `Readseq <http://www.ebi.ac.uk/readseq>`_ service

    ::

        >>> from bioservices import *
        >>> s = Readseq()


    """
    _url = "http://www.ebi.ac.uk/Tools/services/soap/readseq?wsdl"
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose:

        """
        super(Readseq, self).__init__(name="Readseq", url=Readseq._url, verbose=verbose)
        self._parameters = None

    def _get_parameter(self):
        if self._parameters is None:
            self._parameters = self.get_parameters()
        return self._parameters
    parameters = property(_get_parameter, doc="Get list of parameter names")

    def get_parameters(self):
        """Get a list of the parameter names.

        :returns: a list of strings giving the names of the parameters.

        """
        parameters = self.serv.getParameters().id
        return parameters

    def get_parameter_details(self, parameter):
        """Get details of a specific parameter.

        :param str parameter: identifier/name of the parameter to fetch details of.
        :return: a data structure describing the parameter and its values.

        """
        self.devtools.check_param_in_list(parameter, self.parameters)
        return self.serv.getParameterDetails(parameter)

    def run(self, email, title, **kargs):
        """Submit a job to the service.

        :param str email: user e-mail address. 
        :param str title: job title.
        :param params: parameters for the tool as returned by :meth:`get_parameter_details`.
        :return: string containing the job identifier (jobId).
        """
        for k in kargs.keys():
            self.devtools.check_param_in_list(k, self.parameters)
        for k in ['inputformat','outputformat', 'outputcase', 'reverse', 'degap',
                'feature', 'fthandle']:
            if k in kargs.keys():
                value = kargs.get(k)

                valid_values = [x.value for x in self.get_parameter_details(k).values.value]

                self.devtools.check_param_in_list(str(value), valid_values)

        jobid = self.serv.run(email, title, kargs)
        self._jobid = jobid
        return jobid

    def get_status(self, jobid=None):
        """Get the status of a submitted job.

        :param str jobid: job identifier.
        :return: string containing the status.

        The values for the status are:
    
        - RUNNING: the job is currently being processed.
        - FINISHED: job has finished, and the results can then be retrieved.
        - ERROR: an error occurred attempting to get the job status.
        - FAILURE: the job failed.
        - NOT_FOUND: the job cannot be found.


        """
        return self.serv.getStatus(jobid)

    def get_result_types(self, jobid):
        """Get the available result types for a finished job.

        :param str jobid: job identifier.
        :return: a list of wsResultType data structures describing the available result types.
        """
        return self.serv.getResultTypes(jobid)

    def get_result(self, jobid, parameters=None):
        """Get the result of a job of the specified type.

        :param str jobid: job identifier.
        :param parameters: optional list of wsRawOutputParameter used to 
            provide additional parameters for derived result types.
        :return: the result data for the specified type, base64 encoded. 
            Depending on the SOAP library and programming language used the 
            result may be returned in decoded form. For some result types 
            (e.g. images) this will be binary data rather than a text string.
        """
        if self.get_status(jobid) != 'FINISHED':
            self.logging.warning("Your job is not finished yet. Try again.")
            return
        type_ = self.get_result_types(jobid).type[0].identifier
        res = self.serv.getResult(jobid, type_)

        res = res.decode("base64")
        return res

"""
The input parameters for the job:

Attribute   Type    Description
inputformat     int     sequence format for input data
outputformat    int     sequence format for output data
outputcase  string  character case for output sequence data
reverse     boolean     output reverse complement of input nucleotide sequence
degap   string  remove gap symbols from sequence
transymbol  string  replace specified base/residue symbol(s) in input sequence with specified base/residue symbol(s)
feature     string  list of selected features to process
fthandle    string  action to perform on selected features
subrange    string  region of input sequence on which feature processing is performed
sequence    string  input sequence data to process

More detailed information about each parameter, including valid values can be obtained using the getParameterDetails(parameterId) operation.
wsParameterDetails

Descriptive information about a tool parameter. Returned by getParameterDetails(parameterId).
Attribute   Type    Description
name    string  Name of the parameter.
description     string  Description of the parameter, suitable for use in option help interfaces.
type    string  Data type of the parameter.
values  list of wsParameterValue    Optional list of valid values for the option.
wsParameterValue

Description of a tool parameter value. Used in wsParameterDetails.
Attribute   Type    Description
label   string  Display name of the value, for use in interfaces.
value   string  String representation of the value to be passed to the tool parameter.
defaultValue    boolean     Flag indicating if this value is the default.
properties  list of wsProperty  Optional list of key/value pairs providing further information.
wsProperty

Properties of a tool parameter value. Used in wsParameterValue.
Attribute   Type    Description
key     string  Property name.
value   string  Property value.
wsRawOutputParameter

Additional parameters passed when requesting a result. See getResult(jobId, type, parameters).
Attribute   Type    Description
name    string  Parameter name.
value   list of string  Parameter value.
wsResultType

Description of a result type. Returned by getResultTypes(jobId).
Attribute   Type    Description
identifier  string  Identifier for the result type. Passed as type to getResult(jobId, type, parameters).
label   string  Display name for use in user interfaces.
description     string  Description of the result type, for use in help interfaces.
mediaType   string  MIME type of the returned data.
fileSuffix  string  Suggested suffix for file name, if writing data to disk. 
"""



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$

"""This module provides a class :class:`ChEBI` 

.. topic:: What is ChEBI

    :URL:  https://www.ebi.ac.uk/chebi/init.do
    :WSDL: http://www.ebi.ac.uk/webservices/chebi/2.0/webservice


    .. highlights::

        "The database and ontology of Chemical Entities of Biological Interest

        -- From ChEBI web page June 2013


"""
from bioservices import WSDLService


class ChEBI(WSDLService):
    """Interface to `ChEBI <http://www.ebi.ac.u.k/chebi/init.do>`_


        >>> from bioservices import *
        >>> ch = ChEBI()
        >>> res = ch.getCompleteEntity("CHEBI:27732")
        >>> res.smiles
        CN1C(=O)N(C)c2ncn(C)c2C1=O

    """
    _url = "http://www.ebi.ac.uk/webservices/chebi/2.0/webservice?wsdl"
    def __init__(self, verbose=False):
        """.. rubric:: Constructor

        :param bool verbose:

        """
        super(ChEBI, self).__init__(name="ChEBI", url=ChEBI._url, 
            verbose=verbose)

    def getCompleteEntity(self, chebiId):
        """Retrieves the complete entity including synonyms, database links and
chemical structures, using the ChEBI identifier.

        :param str chebiId: a valid ChEBI identifier (string)
        :return: an object containing fields such as mass, names, smiles

        ::

            >>> from bioservices import *
            >>> ch = ChEBI()
            >>> res = ch.getCompleteEntity("CHEBI:27732")
            >>> res.mass
            194.19076

        The returned structure is the raw object returned by the API.
        You can extract names from other sources for instance::

            >>> [x[0] for x in res.DatabaseLinks if x[1].startswith("KEGG")]
            [C07481, D00528]
            >>> [x[0] for x in res.DatabaseLinks if x[1].startswith("ChEMBL")]
            [116485]

        .. seealso:: :meth:`conv`, :meth:`getCompleteEntity`
        """
        res = self.serv.getCompleteEntity(chebiId)
        # the output in res are made of objects that are cast into strings
        #res = [str(x) for x in res] 
        return res

    def conv(self, chebiId, target):
        """Calls :meth:`getCompleteEntity` and returns the identifier of a given database


        :param str chebiId: a valid ChEBI identifier (string)
        :param target: the identifier of the database
        :return: the identifier

        ::

            >>> ch.conv("CHEBI:10102", "KEGG COMPOUND accession")
            ['C07484']

        """
        res = self.serv.getCompleteEntity(chebiId)
        db = [x[1] for x in res.DatabaseLinks]
        if target not in db:
            raise ValueError("valid database target are %s" % db)
        conv = [str(x[0]) for x in res.DatabaseLinks if x[1] == target]
        return conv

    def getLiteEntity(self, search, searchCategory="ALL", maximumResults=200, stars="ALL"):
        """Retrieves list of entities containing the ChEBI ASCII name or identifier

        :param search: search string or category. 
        :param searchCategory: filter with category. Can be ALL, 
        :param int maximumResults: (default is 200)
        :param str stars: filters that can be set to "TWO ONLY", "ALL", "THREE ONLY"

        The input parameters are a search string and a search category. If the search
        category is null then it will search under all fields. The search string accepts
        the wildcard character "*" and also unicode characters. You can get maximum
        results upto 5000 entries at a time.

        ::

            >>> ch.getLiteEntity("CHEBI:27732")
            [(LiteEntity){
               chebiId = "CHEBI:27732"
               chebiAsciiName = "caffeine"
               searchScore = 4.77
               entityStar = 3
             }]
            >>> res = ch.getLiteEntity("caffeine")
            >>> res = ch.getLiteEntity("caffeine", maximumResults=10)
            >>> len(res)
            10


        .. seealso:: :meth:`getCompleteEntity`
        """
        self.devtools.check_param_in_list(searchCategory, ["ALL", "SMILES", "CHEBI ID", 
            "CHEBI NAME", "DEFINITION", "ALL NAMES", "IUAPC", "MASS", 
            "FORMULA", "INCHI", "INCHI KEY"])
        res = self.serv.getLiteEntity(search, searchCategory, maximumResults, stars)
        if len(res):
            return res[0]
        else:
            return res

    def getUpdatedPolymer(self, chebiId):
        """Returns the UpdatedPolymer object

        :param str chebiId:
        :param str chebiId: a valid ChEBI identifier (string)
        :return: an object with information as described below.

        The object contains the updated 2D MolFile structure, GlobalFormula 
        string containing the formulae for each repeating-unit, the GlobalCharge 
        string containing the charge on individual repeating-units and the 
        primary ChEBI ID of the polymer, even if the secondary Identifier was passed
        to the web-service.

        """
        res = self.serv.getUpdatedPolymer(chebiId)
        return res

    def getCompleteEntityByList(self, chebiIdList=[]):
        """Given a list of ChEBI accession numbers, retrieve the complete Entities. 

        The maximum size of this list is 50. 

        .. seealso:: :meth:`getCompleteEntity`
        """
        res = self.serv.getCompleteEntityByList(chebiIdList)
        return res

    def getOntologyParents(self, chebiId):
        """Retrieves the ontology parents of an entity including the relationship type

        :param str chebiId: a valid ChEBI identifier (string)

        """
        res = self.serv.getOntologyParents(chebiId)
        return res

    def getOntologyChildren(self, chebiId):
        """Retrieves the ontology children of an entity including the relationship type

        :param str chebiId: a valid ChEBI identifier (string)

        """
        res = self.serv.getOntologyChildren(chebiId)
        return res

    def getAllOntologyChildrenInPath(self, chebiId, relationshipType,
            onlyWithChemicalStructure=False):
        """Retrieves the ontology children of an entity including the relationship type

        :param str chebiId: a valid ChEBI identifier (string)
        :param str relationshipType: one of "is a", "has part", "has role", 
            "is conjugate base of", "is conjugate acid of", "is tautomer of"
            "is enantiomer of", "has functional parent" "has parent hybride"
            "is substituent group of"

        ::

            >>> ch.getAllOntologyChildrenInPath("CHEBI:27732", "has part")

        """
        self.devtools.check_param_in_list(relationshipType, 
             ["is a", "has part", "has role", 
            "is conjugate base of", "is conjugate acid of", "is tautomer of",
            "is enantiomer of", "has functional parent", "has parent hybride",
            "is substituent group of"])
        res = self.serv.getAllOntologyChildrenInPath(chebiId, relationshipType, 
            onlyWithChemicalStructure)
        return res

    def getStructureSearch(self, structure, mode="MOLFILE",
            structureSearchCategory="SIMILARITY", totalResults=50,
            tanimotoCutoff=0.25):

        """Does a substructure, similarity or identity search using a structure.

        :param str structure: the input structure
        :param str mode:  type of input (MOLFILE, SMILES, CML" (note that 
            the API uses type but this is a python keyword)
        :param str structureSearchCategory: category of the search. Can be
            "SIMILARITY", "SUBSTRUCTURE", "IDENTITY"
        :param int totalResults: limit the number of results to 50 (default)
        :param tanimotoCuoff: limit results to scores higher than this 
            parameter

        ::

            >>> ch = ChEBI()
            >>> smiles = ch.getCompleteEntity("CHEBI:27732").smiles
            >>> ch.getStructureSearch(smiles, "SMILES", "SIMILARITY", 3, 0.25)
        """

        self.devtools.check_param_in_list(structureSearchCategory, 
                ["SIMILARITY", "SUBSTRUCTURE","IDENTITY"])
        self.devtools.check_param_in_list(mode, ["MOLFILE", "SMILES", "CML"])

        res = self.serv.getStructureSearch(structure, mode, 
            structureSearchCategory, totalResults, tanimotoCutoff)
        return res



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      https://github.com/cokelaer/bioservices
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  source: http://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""Modules with common tools to access web resources"""
from __future__ import print_function
import os
import sys
import socket
import platform

from bioservices.settings import BioServicesConfig

# fixing compatiblity python 2 and 3 related to merging or urllib and urllib2 in python 3
try:
    #python 3
    from urllib.request import urlopen
    from urllib.parse import urlparse, urlencode
    from urllib.error import HTTPError
    from urllib.request import Request
except:
    from urllib import urlencode
    from urllib2  import urlopen, Request, HTTPError

# fixing compatibility issue of input/raw_input
if 'raw_input' in __builtins__: input = raw_input

# This is a hack in case suds is already installed.
# Indded, we want suds_jurko instead
sys.path = [x for x in sys.path if 'suds-' not in x]


from easydev import Logging, DevTools


__all__ = ["Service", "WSDLService", "RESTService",
           "BioServicesError", "REST"]


class BioServicesError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Service(Logging):
    """Base class for WSDL and REST classes

    .. seealso:: :class:`REST`, :class:`WSDLService`
    """

    #: some useful response codes
    response_codes = {
        200: 'OK',
        201: 'Created',
        400: 'Bad Request. There is a problem with your input',
        404: 'Not found. The resource you requests does not exist',
        406: "Not Acceptable. Usually headers issue",
        410:  'Gone. The resource you requested was removed.',
        415: "Unsupported Media Type",
        500: 'Internal server error. Most likely a temporary problem',
        503: 'Service not available. The server is being updated, try again later'
        }

    def __init__(self, name, url=None, verbose=True, requests_per_sec=3):
        """.. rubric:: Constructor

        :param str name: a name for this service
        :param str url: its URL
        :param bool verbose: prints informative messages if True (default is
            True)
        :param requests_per_sec: maximum number of requests per seconds
            are restricted to 3. You can change that value. If you reach the
            limit, an error is raise. The reason for this limitation is
            that some services (e.g.., NCBI) may black list you IP. 
            If you need or can do more (e.g., ChEMBL does not seem to have
            restrictions), change the value. You can also have several instance
            but again, if you send too many requests at the same, your future
            requests may be retricted. Currently implemented for REST only


        All instances have an attribute called :attr:`~Service.logging` that
        is an instanceof the :mod:`logging` module. It can be used to print
        information, warning, error messages::

            self.logging.info("informative message")
            self.logging.warning("warning message")
            self.logging.error("error message")

        The attribute :attr:`~Service.debugLevel`  can be used to set the behaviour
        of the logging messages. If the argument verbose is True, the debugLebel
        is set to INFO. If verbose if False, the debugLevel is set to WARNING.
        However, you can use the :attr:`debugLevel` attribute to change it to
        one of DEBUG, INFO, WARNING, ERROR, CRITICAL. debugLevel=WARNING means
        that only WARNING, ERROR and CRITICAL messages are shown.

        """
        super(Service, self).__init__(level=verbose)
        self.requests_per_sec = requests_per_sec

        self._url = url
        try:
            if self.url is not None:
                urlopen(self.url)
        except Exception as err:
            self.logging.warning("The URL (%s) provided cannot be reached." % self.url)
        self.name = name
        self._easyXMLConversion = True

        # used by HGNC where some XML contains non-utf-8 characters !!
        # should be able to fix it with requests once HGNC works again
        #self._fixing_unicode = False
        #self._fixing_encoding = "utf-8"

        self.devtools = DevTools()
        self.settings = BioServicesConfig()
    
    def _get_caching(self):
        return self.settings.params['cache.on'][0]
    def _set_caching(self, caching):
        self.devtools.check_param_in_list(caching, [True, False])
        self.settings.params['cache.on'][0] = caching
        # reset the session, which will be automatically created if we
        # access to the session attribute
        self._session = None
    CACHING = property(_get_caching, _set_caching)

    def _get_url(self):
        return self._url

    def _set_url(self, url):
        # something more clever here to check the URL e.g. starts with http
        if url is not None:
            url = url.rstrip("/")
            self._url = url
    url = property(_get_url, _set_url, doc="URL of this service")

    def _get_easyXMLConversion(self):
        return self._easyXMLConversion

    def _set_easyXMLConversion(self, value):
        if isinstance(value, bool) is False:
            raise TypeError("value must be a boolean value (True/False)")
        self._easyXMLConversion = value
    easyXMLConversion = property(_get_easyXMLConversion,
            _set_easyXMLConversion, 
            doc="""If True, xml output from a request are converted to easyXML object (Default behaviour).""")

    def easyXML(self, res):
        """Use this method to convert a XML document into an
            :class:`~bioservices.xmltools.easyXML` object

        The easyXML object provides utilities to ease access to the XML
        tag/attributes.

        Here is a simple example starting from the following XML

        .. doctest::

            >>> from bioservices import *
            >>> doc = "<xml> <id>1</id> <id>2</id> </xml>"
            >>> s = Service("name")
            >>> res = s.easyXML(doc)
            >>> res.findAll("id")
            [<id>1</id>, <id>2</id>]

        """
        from bioservices import xmltools
        return xmltools.easyXML(res)


    def __str__(self):
        txt = "This is an instance of %s service" % self.name
        return txt

    def pubmed(self, Id):
        """Open a pubmed Id into a browser tab

        :param Id: a valid pubmed Id in string or integer format.

        The URL is a concatenation of the pubmed URL
        http://www.ncbi.nlm.nih.gov/pubmed/ and the provided Id.

        """
        url = "http://www.ncbi.nlm.nih.gov/pubmed/"
        import webbrowser
        webbrowser.open(url + str(Id))

    def on_web(self, url):
        """Open a URL into a browser"""
        import webbrowser
        webbrowser.open(url)

    def save_str_to_image(self, data, filename):
        """Save string object into a file converting into binary"""
        with open(filename,'wb') as f:
            import binascii
            try:
                #python3
                newres = binascii.a2b_base64(bytes(data, "utf-8"))
            except:
                newres = binascii.a2b_base64(data)
            f.write(newres)


class WSDLService(Service):
    """Class dedicated to the web services based on WSDL/SOAP protocol.

    .. seealso:: :class:`RESTService`, :class:`Service`

    """
    _service = "WSDL"

    def __init__(self, name, url, verbose=True, cache=False):
        """.. rubric:: Constructor

        :param str name: a name e.g. Kegg, Reactome, ...
        :param str url: the URL of the WSDL service
        :param bool verbose: prints informative messages

        The :attr:`serv` give  access to all WSDL functionalities of the service.

        The :attr:`methods` is an alias to self.serv.methods and returns
        the list of functionalities.

        """
        super(WSDLService, self).__init__(name, url, verbose=verbose)

        self.logging.info("Initialising %s service (WSDL)" % self.name)
        self.CACHING = cache

        try:
            #: attribute to access to the methods provided by this WSDL service
            from suds.client import Client
            from suds.cache import ObjectCache
            oc = ObjectCache(self.settings.user_config_dir, days=0)
            if self.CACHING is True:
                self.suds = Client(self.url, cache=oc, cachingpolicy=1)
            else:
                self.suds = Client(self.url)
            # reference to the service
            self.serv = self.suds.service
            self._update_settings()
        except Exception :
            self.logging.error("Could not connect to the service %s " % self.url)
            raise Exception

    def _update_settings(self):
        self.TIMEOUT = self.settings.TIMEOUT

    def wsdl_methods_info(self):
        methods = self.suds.wsdl.services[0].ports[0].methods.values()
        for method in methods:
            try:
                print('%s(%s) ' % (
                    method.name,
                    ', '.join('type:%s: %s - element %s' %
                            (part.type, part.name, part.element) for part in
                            method.soap.input.body.parts)))
            except:
                print(method)
    def _get_methods(self):
        return [x.name for x in
                self.suds.wsdl.services[0].ports[0].methods.values()]
    wsdl_methods = property(_get_methods, 
            doc="returns methods available in the WSDL service")

    def wsdl_create_factory(self, name, **kargs):
        params = self.suds.factory.create(name)

        # e.g., for eutils
        if "email" in dict(params).keys():
            params.email = self.settings.params['user.email'][0]

        if "tool" in dict(params).keys():
            import bioservices
            params.tool = "BioServices, " + bioservices.__version__

        for k,v in kargs.items():
            from suds import sudsobject
            keys = sudsobject.asdict(params).keys()
            if k in keys:
                params[k] = v
            else:
                msg = "{0} incorrect. Correct ones are {1}"
                self.logging.error(msg.format(k, keys))
        return params

    def _get_timeout(self):
        return self.suds.options.timeout
    def _set_timeout(self, value):
        self.suds.set_options(timeout=value)
        self.settings.TIMEOUT = value
    TIMEOUT = property(_get_timeout, _set_timeout)


class RESTbase(Service):
    _service = "REST"
    def __init__(self, name, url=None, verbose=True):
        super(RESTbase, self).__init__(name, url, verbose=verbose)
        self.logging.info("Initialising %s service (REST)" % self.name)
        self.last_response = None

    def http_get(self):
        # should return unicode
        raise NotImplementedError

    def http_post(self):
        raise NotImplementedError

    def http_put(self):
        raise NotImplementedError

    def http_delete(self):
        raise NotImplementedError


class RESTService(RESTbase):
    """Class to manipulate REST service

    You can request an URL with this class that also inherits from
    :class:`Service`.


    For debugging:

    * last_response contains

    """
    def __init__(self, name, url=None, verbose=True):
        """.. rubric:: Constructor

        :param str name: a name e.g. Kegg, Reactome, ...
        :param str url: the URL of the REST service
        :param str debugLevel: logging level. See :class:`Service`.

        """
        super(RESTService, self).__init__(name, url, verbose=verbose)
        self.logging.error("Please use REST class instead of RESTService")
        self.trials = 5
        self.timesleep = 1

    def getUserAgent(self):
        self.logging.info('getUserAgent: Begin')
        try:
            import urllib2
            urllib_agent = 'Python-urllib/%s' % urllib2.__version__
        except:
            import urllib
            urllib_agent = 'Python-urllib/%s' % urllib.__version__
        #clientRevision = ''
        clientVersion = ''
        user_agent = 'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
            clientVersion, os.path.basename(__file__),
            platform.python_version(), platform.system(),
            urllib_agent
        )
        self.logging.info('getUserAgent: user_agent: ' + user_agent)
        self.logging.info('getUserAgent: End')
        return user_agent

    def request(self, path, format="xml", baseUrl=True):
        """Send a request via an URL to the web service.

        :param str path: the request will be formed as self.url+/+path
        :param str format: If the expected output is in XML
            format then it will be converted with :meth:`easyXML`. If the
            returned document is not in XML, format should be set to any other
            value.
        :param str baseUrl: By default, the path argument is appended to the
            :attr:`url` attribute (the main REST URL). However, sometimes, you
            would prefer to provide the entire URL yourself (e.g. in psicquic service)
            If so, set this baseUrl argument to False.

        .. note:: this is a HTTP GET request

        .. seealso:: for developers see also the :meth:`_request_timeout`
            if the site is down or busy.

        .. note:: you can set the timeout of the connection, which is 1000
            seconds by default by changing the :attr:`timeout`.
        """
        for i in range(0, self.trials):
            res = self._get_request(path, format=format, baseUrl=baseUrl)
            if res is not None:
                break
            import time
            self.logging.warning("request seemed to have failed.")
            if i!=self.trials-1:
                print("Trying again trial {}/{}".format(i+1, self.trials))
            time.sleep(self.timesleep)
        return res

    def http_get(self, path, format="xml", baseUrl=True):
        return self.request(path, format=format, baseUrl=baseUrl)

    def _get_request(self, path, format="xml", baseUrl=True):
        if path.startswith(self.url):
            url = path
        elif baseUrl is False:
            url = path
        else:
            url = self.url + "/" +  path

        self.logging.debug("REST.bioservices.%s request begins" % self.name)
        self.logging.debug("--Fetching url=%s" % url)

        if len(url)> 2000:
            print(url)
            raise ValueError("URL length (%s) exceeds 2000. Please use a differnt URL" % len(url))

        try:
            res = urlopen(url).read()
            if format=="xml":
                if self.easyXMLConversion:
                    #logging.warning("--Conversion to easyXML"),
                    try:
                        res = self.easyXML(res)
                    except Exception as err:
                        self.logging.warning(err.message)
                        self.logging.warning("--Conversion to easyXML failed. returns the raw response"),
            self.last_response = res
            return res
        except socket.timeout:
            self.logging.warning("Time out. consider increasing the timeout attribute (currently set to {})".format(self.timeout))
        except Exception as err:
            self.logging.debug(err.message)
            self.logging.debug("An exception occured while reading the URL")
            self.logging.debug(url)
            self.logging.debug("Error caught within bioservices. Invalid requested URL ? ")


    def requestPost(self, requestUrl, params, extra=None):
        """request with a POST method.

        :param str requestUrl: the entire URL to request
        :param dict params: the dictionary of parameter/value pairs
        :param str extra: an additional string to add after the params if
            needed. Could be usefule if a parameter/value can not be added to the
            dictionary. For instance is a parameter has several values
            Solved in requests module.
        .. todo:: parameter paranName with a list of values [v1,v2] can be interpreted as
            paramName=v1&paramName=v2

        .. note:: this is a HTTP POST request
        .. note:: use only by ::`ncbiblast` service so far.
        """
        requestData = urlencode(params)
        print(requestData)
        if extra is not None:
            requestData += extra
        # Concatenate the two parts.
        # Errors are indicated by HTTP status codes.
        try:
            # Set the HTTP User-agent.
            user_agent = self.getUserAgent()
            http_headers = {'User-Agent': user_agent}
            print(requestUrl)
            req = Request(requestUrl, None, http_headers)
            # Make the submission (HTTP POST).
            print(req)
            reqH = urlopen(req, requestData)
            jobId = reqH.read()
            reqH.close()
        except HTTPError as err:
            # Trap exception and output the document to get error message.
            print(sys.stderr, ex.read())
            raise
        return jobId

    def urlencode(self, params):
        """Returns a string compatible with a URL request.

        :param dict params: a dictionary. Keys are parameters.

        The pair of key/value are converted into a single string by concatenated
        the "&key=value" string for each key/value in the dictionary.

        ::

            >>> params = {'a':1, 'b':2}
            >>> urlencode(params)
            "a=1&b=2"

        .. note:: returns "a=1&b=2" or "b=2&a=1" since dictionary are not ordered. Note
            that the first parameter is not preceded by a & sign that you will need
            to add.

        """
        if isinstance(params, dict)is False:
            raise TypeError("Params must be a dictionary.")
        postData = urlencode(params)
        return postData


import requests         # replacement for urllib2 (2-3 times faster)
from requests.models import Response
import requests_cache   # use caching wihh requests
#import grequests        # use asynchronous requests with gevent
# Note that grequests should be imported after requests_cache. Otherwise,
# one should use a session instance when calling grequests.get, which we do
# here below


class REST(RESTbase):
    """

    The ideas (sync/async) and code using requests were inspired from the chembl
    python wrapper but significantly changed.

    Get one value::

        >>> from bioservices import REST
        >>> s = REST("test", "https://www.ebi.ac.uk/chemblws")
        >>> res = s.get_one("targets/CHEMBL2476.json", "json")
        >>> res['organism']
        u'Homo sapiens'

    The caching has two major interests. First one is that it speed up requests if
    you repeat requests. ::


        >>> s = REST("test", "https://www.ebi.ac.uk/chemblws")
        >>> s.CACHING = True
        >>> # requests will be stored in a local sqlite database
        >>> s.get_one("targets/CHEMBL2476")
        >>> # Disconnect your wiki and any network connections.
        >>> # Without caching you cannot fetch any requests but with
        >>> # the CACHING on, you can retrieve previous requests:
        >>> s.get_one("targets/CHEMBL2476")


    Advantages of requests over urllib

    requests length is not limited to 2000 characters
    http://www.g-loaded.eu/2008/10/24/maximum-url-length/


    There is no need for authentication if the web services available
    in bioservices except for a few exception. In such case, the username and
    password are to be provided with the method call. However, 
    in the future if a services requires authentication, one can set the
    attribute :attr:`authentication` to a tuple::

        s = REST()
        s.authentication = ('user', 'pass')
    """
    content_types = {
        'bed': 'text/x-bed',
        'default': "application/x-www-form-urlencoded",
        'gff3': 'text/x-gff3',
        'fasta': 'text/x-fasta',
        'json': 'application/json',
        "jsonp": "text/javascript",
        "nh": "text/x-nh",
        'phylip': 'text/x-phyloxml+xml',
        'phyloxml': 'text/x-phyloxml+xml',
        'seqxml': 'text/x-seqxml+xml',
        'txt': 'text/plain',
        'text': 'text/plain',
        'xml': 'application/xml',
        'yaml': 'text/x-yaml'
    }
    #special_characters = ['/', '#', '+']

    def __init__(self, name, url=None, verbose=True, cache=False):
        super(REST, self).__init__(name, url, verbose=verbose)
        bspath = self.settings.user_config_dir
        self.CACHE_NAME = bspath + os.sep + self.name + "_bioservices_db"

        self._session = None

        self.settings.params['cache.on'][0] = cache

        if self.CACHING:
            #import requests_cache
            self.logging.info("Using local cache %s" % self.CACHE_NAME)
            requests_cache.install_cache(self.CACHE_NAME)

    def delete_cache(self):
        import os
        cache_file = self.CACHE_NAME + '.sqlite'
        if os.path.exists(cache_file):
            msg = "You are about to delete this bioservices cache: %s. Proceed? (y/[n]) "
            res = input(msg % cache_file)
            if res == "y":
                os.remove(cache_file)
                print("Removed cache")
            else:
                print("Reply 'y' to delete the file")

    def clear_cache(self):
        from requests_cache import clear
        clear()

    def _get_session(self):
        if self._session is None:
            if self.CACHING is True:
                self._session = self._create_cache_session()
            else:
                self._session = self._create_session()
        return self._session
    session = property(_get_session)

    def _create_session(self):
        """Creates a normal session using HTTPAdapter

        max retries is defined in the :attr:`MAX_RETRIES`
        """
        self.logging.debug("Creating session (uncached version)")
        self._session = requests.Session()
        adapter = requests.adapters.HTTPAdapter(max_retries=self.settings.MAX_RETRIES)
        #, pool_block=True does not work with asynchronous requests
        self._session.mount('http://', adapter)
        self._session.mount('https://', adapter)
        return self._session

    def _create_cache_session(self):
        """Creates a cached session using requests_cache package"""
        self.logging.debug("Creating session (cache version)")
        if not self._session:
            #import requests_cache
            self.logging.debug("No cached session created yet. Creating one")
            self._session = requests_cache.CachedSession(self.CACHE_NAME,
                         backend='sqlite', fast_save=self.settings.FAST_SAVE)
        return self._session

    def _get_timeout(self):
        return self.settings.TIMEOUT
    def _set_timeout(self, value):
        self.settings.TIMEOUT = value
    TIMEOUT = property(_get_timeout, _set_timeout)

    def _process_get_request(self, url, session, frmt, data=None, **kwargs):
        try:
            res = session.get(url, **kwargs)
            self.last_response = res
            res = self._interpret_returned_request(res, frmt)
            return res
        except Exception:
            return None

    def _interpret_returned_request(self, res, frmt):
        # must be a Response
        if isinstance(res, Response) is False:
            return res
        # if a response, there is a status code that should be ok
        if not res.ok:
            reason = res.reason
            self.logging.warning("status is not ok with {0}".format(reason))
            return res.status_code
        if frmt == "json":
            try:
                return res.json()
            except:
                return res
        # finally
        return res.content

    def _apply(self, iterable, fn, *args, **kwargs):
        return [fn(x, *args, **kwargs) for x in iterable if x is not None]

    def _get_async(self, keys, frmt='json', params={}):
        # does not work under pyhon3 so local import
        import grequests
        session = self._get_session()
        try:
            # build the requests
            urls = self._get_all_urls(keys, frmt)
            self.logging.debug("grequests.get processing")
            rs = (grequests.get(url, session=session, params=params)  for key,url in zip(keys, urls))
            # execute them
            self.logging.debug("grequests.map call")
            ret = grequests.map(rs, size=min(self.settings.CONCURRENT, len(keys)))
            self.last_response = ret
            self.logging.debug("grequests.map call done")
            return ret
        except Exception as err:
            self.logging.warning("Error caught in async. " + err.message)
            return []

    def _get_all_urls(self, keys, frmt=None):
        return ('%s/%s' % (self.url, query) for query in keys)

    def get_async(self, keys, frmt='json', params={}, **kargs):
        ret = self._get_async(keys, frmt, params=params, **kargs)
        return self._apply(ret, self._interpret_returned_request, frmt)

    def get_sync(self, keys, frmt='json', **kargs):
        return [self.get_one(key, frmt=frmt, **kargs) for key in keys]

    def http_get(self, query, frmt='json', params={}, **kargs):
        """

        * query is the suffix that will be appended to the main url attribute.
        * query is either a string or a list of strings.
        * if list is larger than ASYNC_THRESHOLD, use asynchronous call.


        """
        if isinstance(query, list) and len(query) > self.settings.ASYNC_THRESHOLD:
            self.logging.debug("Running async call for a list")
            return self.get_async(query, frmt, params=params, **kargs)
        if isinstance(query, list) and len(query) <= self.settings.ASYNC_THRESHOLD:
            self.logging.debug("Running sync call for a list")
            return [self.get_one(key, frmt, params=params, **kargs) for key in query]
            #return self.get_sync(query, frmt)
        # OTHERWISE
        self.logging.debug("Running http_get (single call mode)")
        #return self.get_one(**{'frmt': frmt, 'query': query, 'params':params})
        return self.get_one(query, frmt, params=params, **kargs)

    def get_one(self, query=None, frmt='json', params={}, **kargs):
        """

        if query starts with http:// do not use self.url
        """
        if query is None:
            url = self.url
        else:
            if query.startswith("http"):
                # assume we do want to use self.url
                url = query
            else:
                url = '%s/%s' % (self.url, query)
        self.logging.debug(url)
        try:
            kargs['params'] = params
            kargs['timeout'] = self.TIMEOUT
            # Used only in biomart with cosmic database
            # See doc/source/biomart.rst for an example
            if hasattr(self, 'authentication'):
                kargs['auth'] = self.authentication

            #res = self.session.get(url, **{'timeout':self.TIMEOUT, 'params':params})
            res = self.session.get(url, **kargs)

            self.last_response = res
            res = self._interpret_returned_request(res, frmt)
            try:
                # for python 3 compatibility
                res = res.decode()
            except:
                pass
            return res
        except Exception as err:
            print(err)
            print("""Issue. Maybe your current timeout is {0} is not sufficient. 
Consider increasing it with settings.TIMEOUT attribute""".format(self.settings.TIMEOUT))

    def http_post(self, query, params=None, data=None,
                    frmt='xml', headers=None, files=None, **kargs):
        # query and frmt are bioservices parameters. Others are post parameters
        # NOTE in requests.get you can use params parameter
        # BUT in post, you use data
        # only single post implemented for now unlike get that can be asynchronous
        # or list of queries
        if headers is None:
            headers = {}
            headers['User-Agent'] = self.getUserAgent()
            headers['Accept'] = self.content_types[frmt]

        self.logging.debug("Running http_post (single call mode)")
        kargs.update({'query':query})
        kargs.update({'headers':headers})
        kargs.update({'files':files})
        kargs.update({'params':params})
        kargs.update({'data':data})
        kargs.update({'frmt':frmt})
        return self.post_one(**kargs)

    def post_one(self, query=None, frmt='json', **kargs):
        self.logging.debug("BioServices:: Entering post_one function")
        if query is None:
            url = self.url
        else:
            url = '%s/%s' % (self.url, query)
        self.logging.debug(url)
        try:
            res = self.session.post(url, **kargs)
            self.last_response = res
            res = self._interpret_returned_request(res, frmt)
            try:
                return res.decode()
            except:
                self.debug("BioServices:: Could not decode the response")
                return res
        except Exception as err:
            print(err)
            return None

    def getUserAgent(self):
        self.logging.info('getUserAgent: Begin')
        urllib_agent = 'Python-requests/%s' % requests.__version__
        #clientRevision = ''
        clientVersion = ''
        user_agent = 'EBI-Sample-Client/%s (%s; Python %s; %s) %s' % (
            clientVersion, os.path.basename(__file__),
            platform.python_version(), platform.system(),
            urllib_agent
        )
        self.logging.info('getUserAgent: user_agent: ' + user_agent)
        self.logging.info('getUserAgent: End')
        return user_agent

    def get_headers(self, content='default'):
        """
        :param str content: ste to default that is application/x-www-form-urlencoded
            so that it has the same behaviour as urllib2 (Sept 2014)

        """
        headers = {}
        headers['User-Agent'] = self.getUserAgent()
        headers['Accept'] = self.content_types[content]
        headers['Content-Type'] = self.content_types[content]
        #"application/json;odata=verbose" required in reactome
        #headers['Content-Type'] = "application/json;odata=verbose" required in reactome
        return headers

    def debug_message(self):
        print(self.last_response.content)
        print(self.last_response.reason)
        print(self.last_response.status_code)

    def http_delete(self, query, params=None, 
                    frmt='xml', headers=None,  **kargs):
        kargs.update({'query': query})
        kargs.update({'params': params})
        kargs.update({'frmt': frmt})
        return self.delete_one(**kargs)

    def delete_one(self, query, frmt='json', **kargs):
        self.logging.debug("BioServices:: Entering delete_one function")
        if query is None:
            url = self.url
        else:
            url = '%s/%s' % (self.url, query)
        self.logging.debug(url)
        try:
            res = self.session.delete(url, **kargs)
            self.last_response = res
            res = self._interpret_returned_request(res, frmt)
            try:
                return res.decode()
            except:
                self.debug("BioServices:: Could not decode the response")
                return res
        except Exception as err:
            print(err)
            return None
        except:
            pass



from bioservices import NCBIblast
from nose.plugins.attrib import attr

class test_nciblast(NCBIblast):
    def __init__(self):
        super(test_nciblast, self).__init__(verbose=False)

    def test_param(self):
        self.getParameters()
        assert(len(self.parameters)>0)
        assert(len(self.parameters)>0)

    
    def test_paramdetails(self):
        names = self.parametersDetails("matrix") 
        try:
            names = self.parametersDetails("matrixddddd") 
            assert False
        except:
            assert True

    @attr('slow')
    def test_run(self):

        try:
            self.jobid = self.run(program="blastp", sequence=self._sequence_example,
                stype="protein", database="uniprotkb")
            assert False # missing email argument
        except:
            assert True
        self.jobid = self.run(program="blastp", sequence=self._sequence_example,
            stype="protein", database="uniprotkb", email="cokelaer@ebi.ac.uk",
            matrix="BLOSUM45")
        res = self.getResult(self.jobid, "out")

        res = self.getResultTypes(self.jobid)

    def test_attributse(self):
        self.databases



from bioservices import ENA



def test_ena():


    e = ENA()

    e.get_data('ERA000092', 'xml')
    e.get_data('ERA000092', 'text')
    e.get_data('ERA000092', 'fasta')


    res = e.get_data('ERA000092', 'fasta', fasta_range=[3,63])


    e.get_data('AL513382', frmt='text', expanded=True)


    e.get_data('BN000065', 'text', header=True)

    e.get_data('AM407889.1', 'fasta')
    e.get_data('AM407889.2', 'fasta')

    





from bioservices import Clinvitae



def test_clinvitae():

    c = Clinvitae()
    res = c.query_gene('brca1')
    entry1 = res[0]
    entry1.keys()
    assert entry1['accessionId'] == 'SCV000039520'
    assert entry1['lastEvaluated'] # not a stable test # == '2013-04-03'
    assert entry1['source'] == 'ClinVar'



    res = c.query_gene('NM_198578.3:c.1847A>G')  # returns all entries in LRRK2 gene
    entry1 = res[0]
    assert entry1['accessionId']  == 'SCV000056058'
    assert entry1['lastEvaluated'] == u'2012-09-13'
    assert entry1['source'] == 'ClinVar'

    res = c.all_variants('MUTYH')  # returns all reported variants in MUTYH gene
    assert len(res)>0


    pathogenic = c.get_pathogenic('brca1')  # returns pathogenic or likely pathogenic
    assert len(pathogenic) > 100


    benign = c.get_benign('brca1')  # returns benign or likely benign
    assert len(benign) > 100

    vus = c.get_VUS('brca1')
    assert len(vus) > 100



from bioservices.muscle import MUSCLE
from bioservices import uniprot


def test_muscle():
    m = MUSCLE(verbose=False)
    m.parameters
    m.getParametersDetails("format")


    u = uniprot.UniProt(verbose=False)
    f1 = u.get_fasta("P18812")
    f2 = u.get_fasta("P18813")


    jobid = m.run(frmt="fasta", sequence=f1+f2, email="cokelaer@ebi.ac.uk")
    m.getStatus(jobid)
    m.wait(jobid)

    m.getResultTypes(jobid)
    m.getResult(jobid, 'phylotree')




from bioservices import Reactome
from nose.plugins.attrib import attr



class test_Reactome(object):

    @classmethod
    def setup_class(klass):
        klass.e = Reactome(verbose=True)

    def test_species(self):
        assert len( self.e.get_species()) > 10

    def test_biopax_exporter(self):
        res = self.e.biopax_exporter(109581)

    def test_front_page_items(self):
        res = self.e.front_page_items("homo sapiens")

    def test_highlight_pathway_diagram(self):
        res = self.e.highlight_pathway_diagram(68875, frmt="PNG", genes="CDC2")

    def test_list_by_query(self):
        assert len(self.e.list_by_query("Pathway", name="Apoptosis"))>1

    def test_pathway_diagram(self):
        res = self.e.pathway_diagram(109581, 'XML')
        res = self.e.pathway_diagram(109581, 'PNG')
        res = self.e.pathway_diagram(109581, 'PDF')
        try:
            res = self.e.pathway_diagram(109581, 'XDF')
            assert False
        except:
            assert True

    @attr('slow')
    def test_pathway_hierarchy(self):
        res = self.e.pathway_hierarchy('homo sapiens')
        assert len(res)>10

    def test_pathway_participants(self):
        res = self.e.pathway_participants(109581)
        assert len(res)>0

    def test_pathway_complexes(self):
        res = self.e.pathway_complexes(109581)
        assert len(res)>1
        assert "dbId" in list(res[0].keys())

    def test_query_by_id(self):
        res = self.e.query_by_id("Pathway", 109581)
        assert len(res)>1

    def test_query_by_ids(self):
        res = self.e.query_by_ids("Pathway", 'CDC2')
        assert len(res)>=1

    def test_query_hit_pathways(self):
        assert len(self.e.query_hit_pathways("CDC2"))>1

    def test_pathway_for_entities(self):
        res = self.e.query_pathway_for_entities("CDC2")

    def test_species_list(self):
        res = self.e.species_list()

    @attr('slow')
    def test_sbml_exporter(self):
        res = self.e.SBML_exporter(109581)






from bioservices.intact import IntactComplex



def test_intact():
    i = IntactComplex()
    res = i.details('EBI-1163476')
    assert res['ac'] == 'EBI-1163476'

    res = i.search('ndc80')
    try:
        res = i.search('ndc80', frmt='pandas')
    except:
        pass




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id$
"""This module provides a class :class:`~bioservices.picr.PICR` that allows an
access to the REST interface of the PICR web service. There is also a SOAP web service but we implemented only the REST interface since they both provide access to the same functionalities.

.. topic:: What is PICR ?

    :URL: http://www.ebi.ac.uk/Tools/picr
    :REST: http://www/ebi.ac.uk/Tools/picr/rest
    :Citations: http://www.biomedcentral.com/1471-2105/8/401


    .. highlights::

        "The Protein Identifier Cross-Reference (PICR) service is a web application
        that provides interactive and programmatic (SOAP and REST) access to a mapping
        algorithm based on 100% sequence identity to proteins from over 98 distinct
        source databases. Mappings can be limited by source database, taxonomic ID and
        activity status in the source database. Users can copy/paste or upload files
        containing protein identifiers or sequences in FASTA format to obtain mappings
        using the interactive interface. "

        -- From the PICR home page, Dec 2012


"""
from bioservices.services import REST


__all__ = ["PICR"]
#//the NEWT taxonomy ID to limit the mappings to
#//can be null or a number. Do not specify 0 for null.
#String taxonID = "9606";     //H. Sapiens


class PICR(REST):
    """Interface to the `PICR (Protein Identifier Cross reference) <http://www.ebi.ac.uk/Tools/picr/>`_ service

    .. doctest::

        >>> from bioservices import PICR
        >>> p = PICR()
        >>> res = p.getMappedDatabaseNames() # get list of valid database
        >>> results = p.getUPIForSequence(p._sequence_example, \
                ["IPI", "ENSEMBL", "SWISSPROT"])


    """
    _sequence_example="MDSTNVRSGMKSRKKKPKTTVIDDDDDCMTCSACQSKLVKISDITKVSLDYINTMRGNTLACAACGSSLKLLNDFAS"
    _blastfrag_example="MSVMYKKILYPTDFSETAEIALKHVKAFKTLKAEEVILLHVIDEREIKKRDIFSLLLGVAGLNKSVEEFENELKNKLTEEAKNKMENIKKELEDVGFKVKDIIVVGIPHEEIVKIAEDEGVDIIIMGSHGKTNLKEILLGSVTENVIKKSNKPVLVVKRKNS"
    _accession_example = "P29375"
    _url = "http://www.ebi.ac.uk/Tools/picr/rest"

    def __init__(self, verbose=False, cache=False):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages (default is False)

        """
        super(PICR, self).__init__(name="PICR", url=PICR._url, verbose=verbose, 
                cache=cache)
        self._databases = None

        # the blast call needs time
        self.settings.params['general.timeout'][0] = 100

    def getMappedDatabaseNames(self):
        """Returns names of the databases

        :returns: An XML containing the databases available.

        .. seealso:: :attr:`databases` to obtain a human readable list
        """
        url = "getMappedDatabaseNames"
        res = self.http_get(url, frmt="txt")
        res = self.easyXML(res)
        return res

    def _get_databases(self):
        if self._databases is None:
            res = self.getMappedDatabaseNames()
            self._databases = [a.text for a in res.getchildren()]
        return self._databases
    databases = property(_get_databases, 
        doc="Get a human-readable list of databases (obtained from XML returned by :meth:`getMappedDatabaseNames`)")

    def getUPIForSequence(self, sequence, database, taxid=None,
            onlyactive=True, includeattributes=False):
        """Get Protein identifier given an exact sequence

        :param sequence: the sequence to map [required]
        :param database: the database to map to. At least one database is required, but
            multiple databases can be queried at once.
        :param taxid: the NEWT taxon ID to limit the mappings [optional]
        :param onlyactive: if true, only active mappings will be returned.
            If false, results may include deleted mappings. [optional, default is true]
        :param includeattributes: if true, extra attributes such as sequence and taxon
            IDs will be returned if available. If false, no extra information returned.
            [optional, default is false]

        .. note:: Parameter names are case sensitive.

        .. note:: Some servers, browsers and other clients may have restrictions on the length of
            the query string, so long sequences might cause errors. If this is the case, use
            a POST request rather than a GET.

        .. note:: If a taxid is submitted, includeattributes will be true.

        .. doctest::

            >>> from bioservices import PICR
            >>> p = PICR()
            >>> sequence = p._sequence_example
            >>> databases = ["IPI", "ENSEMBL", "SWISSPROT"]
            >>> results = p.getUPIForSequence(sequence, databases)

        """
        url = "getUPIForSequence"

        # check validity of the database provided
        params = {'sequence':sequence}
        if isinstance(database, str):
            self._checkDBname(database)
            params['database'] = database
        elif isinstance(database,list):
            for d in database:
                self._checkDBname(d)
            # requests.get accepts multidict values, so nothing fancy to do here !
            params['database'] = database[:]
                
        #if taxid!=None or onlyactiveis False or includeattributesis False:
        #    raise NotImplementedError
        if taxid:
            params['taxonid'] = taxid
        if includeattributes is False:
            params['includeattributes'] = 'false'
        if onlyactive is False:
            params['onlyactive'] = "false"

        res = self.http_get(url, frmt="xml", params=params)
        res = self.easyXML(res)
        return res

    def _checkDBname(self, db):
        if db not in self.databases:
            raise ValueError("Provided database name (%s) is not valid. Use database attribute to check the valid names" % db)

    def getUPIForAccession(self, accession, database,
            taxid=None, version=None, onlyactive=True, includeattributes=True):
        """Get Protein identifier given an accession number

        :param str accession:  the accession to map [required]
        :param str version: the version of accession to map [optional]
        :param database: the database to map to (string). At least one database is
            required, but multiple databases can be queried at once using a list.
        :param taxid: the NEWT taxon ID to limit the mappings [optional]
        :param bool onlyactive: if true, only active mappings will be returned. If false,
            results may include deleted mappings. [optional, default is true]
        :param bool includeattributes: if true, extra attributes such as sequence
            and taxon IDs will be returned if available. If false, no extra
            information returned. [optional, default is false]

        .. note:: parameter names are case sensitive

        .. note:: If version is not specified but the accession is of the form P29375.1,
            the accession and version will automatically be split to accession=P29375
            and version-1.

        .. note:: If a taxid is submitted, includeattributes will be true.

        Example:

        ::

            >>> from bioservices import *
            >>> s = PICR()
            >>> s.getUPIForAccession("P29375", ["IPI", "ENSEMBL"])
            >>> s.getUPIForAccession("P29375-1", ["IPI", "ENSEMBL"])
        """

        url =  "getUPIForAccession"
        
        params = {}
        params['accession'] = accession

        if isinstance(database,str):
            self._checkDBname(database)
            params['database'] = database
        elif isinstance(database,list):
            for d in database:
                self._checkDBname(d)
            params['database'] = database[:]
        if taxid:
            params['taxonid'] = taxid
        if includeattributes is False:
            params['includeattributes'] = "false"
        if onlyactive is False:
            params['onlyactive'] = "false"

        res = self.http_get(url, frmt="xml", params=params)
        res = self.easyXML(res)

        return res

    def getUPIForBLAST(self, blastfrag, database,
            taxid=None, version=None, onlyactive=True,
            includeattributes=False, **kargs):
        """Get Protein identifier given a sequence similarity (BLAST)

        :param str blastfrag:  the AA fragment to map [required]
        :param str database: the database to map to. At least one database is
            required, but multiple databases can be queried at once using a list.
        :param taxid: the NEWT taxon ID to limit the mappings [optional]
        :param bool onlyactive: if true, only active mappings will be returned. If false,
            results may include deleted mappings. [optional, default is true]
        :param bool includeattributes: if true, extra attributes such as sequence
            and taxon IDs will be returned if available. If false, no extra
            information returned. [optional, default is false]


        Other options (related to BLAST analysis) can be provided as optional
        argument. See this link for details:

            http://www.ebi.ac.uk/Tools/picr/RESTDocumentation.do

        As an example, you can provide the matrix argument:

        :param str matrix: specifies which protein scoring matrix to use. [optional, defaults to BLOSUM62]

        .. note:: Parameter names are case sensitive.
        .. note:: If version is not specified but the accession is of the form P29375.1,
            the accession and version will automatically be split to accession=P29375
            and version-1.

        .. note:: If a taxid is submitted, includeattributes will be true.


        ::

            >>> res = s.getUPIForBLAST(s._blastfrag_example, "SWISSPROT")
            >>> res = s.getUPIForBLAST(s._blastfrag_example, "SWISSPROT",
                   program="blastp", matrix="BLOSUM80")

        .. todo:: add missing parameters such as filtertype, blast parameters,
            identity Value and so on.
        """
        url = "getUPIForBLAST"
        params = {'blastfrag': blastfrag}

        if isinstance(database,str):
            self._checkDBname(database)
            params['database'] = database
        elif isinstance(database,list):
            for d in database:
                self._checkDBname(d)
                params["database"] = params
        if taxid:
            params["taxonid"] = taxid
        if includeattributes is False:
            params["includeattributes"] = "false"
        if onlyactive is False:
            params["onlyactive"] = "false"

        res = self.http_get(url, frmt="xml", params=params)
        res = self.easyXML(res)

        return res



"""
Parameters:

    BLAST search parameters: These should not be changed unless you are sure that you know what you are doing.
        filtertype - which parameters should the BLAST results be filtered by, valid values are ORGANISM, IDENTITY, ORGANISM_AND_IDENTITY, and NONE. [optional, defaults to NONE]
            identityvalue - the minimum score to accept as a BLAST hit. [required if filtertype is IDENTITY or ORGANISM_AND_IDENTITY]
            identitytaxon - the target organism that BLAST is filtering for. [required if filtertype is ORGANISM or ORGANISM_AND_IDENTITY]
        program - which BLAST program to use for mapping [optional, defaults to blastp]
        alignments - specifies the number of alignments to find pre-filtering (there is probably no need to change this.) [optional, defaults to 100]
        scores - sets the maximum number of scores to display in output. [optional, defaults to 100]
        exp - sets the E-value cutoff. [optional, defaults to 10]
        dropoff - Amount score must drop before extension of hits is halted. [optional, defaults to 0]
        gapopen - sets the gap opening penalty. [optional, defaults to 11]
        gapext - sets the gap extension penalty [optional, defaults to 1]
        gapalign - perform gapped alignment. [optional, defaults to true]
        filter - specifies filters to be used to mask query sequence. See table 3.2.7 for details. [optional, defaults to F, ie none]
        align - sets the alignment view. [optional, defaults to 0]
        stype - sets the type of submission. [optional, defaults to protein]
        blastdb - set the database against which to query the protein fragments. [optional, defaults to uniprotkb]

"""



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id$
"""Interface to the quickGO interface

.. topic:: What is quickGO

    :URL: http://www.ebi.ac.uk/QuickGO/
    :Service: http://www.ebi.ac.uk/QuickGO/WebServices.html

    .. highlights::

        "QuickGO is a fast web-based browser for Gene Ontology terms and
        annotations, which is provided by the UniProt-GOA project at the EBI. "

        -- from QuickGO home page, Dec 2012

"""
from __future__ import print_function

from bioservices.services import REST

__all__ = ["TCGA"]


class TCGA(REST):
    """Interface to the `TCGA`_ service

    DRAFT in progress

    https://wiki.nci.nih.gov/display/TCGA/TCGA+Annotations+Web+Service+User%27s+Guide

    """
    def __init__(self, verbose=False, cache=False):
        """.. rubric:: Constructor

        :param bool verbose: print informative messages.

        """
        super(TCGA,
                self).__init__(url="http://tcga-data.nci.nih.gov",
            name="TCGA", verbose=verbose, cache=cache)

    def search_annotations(self, item, annotationId):
        """Obtain Term information

        """
        params = {'item':item, 'annotationId': annotationId}
        res = self.http_get("annotations/resources/searchannotations/json", 
                frmt="json", params=params)

        return res


    def view_annotations(self):
        raise NotImplementedError








from bioservices import eutils


class test_EUtils(object):
    @classmethod
    def setup_class(klass):
        klass.e = eutils.EUtils(verbose=False)

    def test_taxonomy(self):
        ret = self.e.taxonomy_summary("9606")
        assert ret['9606']['taxid'] == 9606

    def test_snp(self):
        self.e.snp_summary("123")

    def test_databases(self):
        assert "proteinclusters" in self.e.databases

    def test_summary(self):
        ret = self.e.ESummary("taxonomy", "9606,9913")
        assert ret['9606']['taxid'] == 9606
        assert ret['9913']['taxid'] == 9913

        # test another database 
        ret = self.e.ESummary("structure", "10000")
        assert ret['10000']['pdbacc'] == '1CA4'

    def test_espell(self):
        ret = self.e.ESpell(db="omim", term="aasthma+OR+alergy")
        ret = ret.eSpellResult
        print(ret.Query)
        assert ret.Query == 'aasthma OR alergy'
        assert ret.CorrectedQuery == 'asthma or allergy'

    def test_elink(self):
        ret = self.e.ELink(db="pubmed", dbfrom="pubmed", id="20210808",
                           cmd="neighbor_score")

        ret = self.e.ELink(db="pubmed", id="24064416")

        # assert len(ret.LinkSet[0].LinkSetDb[0].Link) > 10

        self.e.ELink(dbfrom="nucleotide", id="48819,7140345", db="protein")

    def test_einfo(self):
        # Use XML
        dbinfo = self.e.EInfo("gtr", retmode='xml')
        dbinfo = self.e.parse_xml(dbinfo, 'EUtilsParser').eInfoResult
        dbinfo = dbinfo.DbInfo
        assert dbinfo.DbName == 'gtr'
        assert dbinfo.MenuName == 'GTR'
        assert dbinfo.Description == 'GTR Database'
        assert dbinfo.FieldList.Field[0].Name == 'ALL'

        # use JSON
        ret = self.e.EInfo("taxonomy")
        ret['count']

        alldbs = self.e.EInfo()
        for db in ['pubmed', 'genome', 'dbvar', 'gene']:
            assert db in alldbs
        assert len(alldbs) > 40  # 52 Aug 2014 but let us be on the safe side

    def test_einfo_pubmed(self):
        ret = self.e.EInfo('pubmed')
        assert ret['dbname'] == 'pubmed'
        assert ret['menuname'] == 'PubMed'
        assert ret['description'] == 'PubMed bibliographic record'
        assert int(ret['count']) > 17905967
        assert ret['lastupdate']
        assert len(ret['fieldlist']) >= 40
        assert ret['fieldlist'][0]['name'] == 'ALL'
        assert ret['fieldlist'][0]['fullname'], 'All Fields'

    def test_gquery(self):
        ret = self.e.EGQuery("asthma")
        [(x.DbName, x.Count) for x in ret.eGQueryResult.ResultItem
         if x.Count != '0']

    def test_efetch(self):
        ret = self.e.EFetch("omim", "269840")
        ret1 = self.e.EFetch("protein", "34577063",
                             retmode="text", rettype="fasta", stand=1)
        ret2 = self.e.EFetch("protein", "34577063",
                             retmode="text", rettype="fasta", stand=2)
        self.e.EFetch("protein", "34577063",
                      retmode="text", rettype="fasta", strand=2,
                      seq_start=10, seq_stop=20)

        ret = self.e.EFetch('pubmed', '12091962,9997',
                            retmode='xml', rettype='abstract')
        # sequences
        res1 = self.e.EFetch("protein", "352, 234",
                             retmode="text", rettype="fasta")
        res2 = self.e.EFetch("protein", ["352", "234"],
                             retmode="text", rettype="fasta")
        res3 = self.e.EFetch("protein", [352, 234],
                             retmode="text", rettype="fasta")

        assert res1 == res2
        assert res2 == res3
        assert res1 == res3

    def test_efetch_gene(self):
        res = self.e.EFetch('gene', 4747, retmode="text")
        assert b'neurofilament' in res
        res = self.e.EFetch('gene', 4747, retmode="xml")

    def test_epost(self):
        ret = self.e.EPost("pubmed", "20210808")
        assert ret['QueryKey']
        assert ret['WebEnv']
        print(ret)



from bioservices.rhea import Rhea
from nose.plugins.attrib import attr


@attr('skip_travis')
class test_rhea(Rhea):
    def __init__(self):
        super(test_rhea, self).__init__(verbose=False)

    def test_search(self):
        r1 = self.search("caffeine")
        r1 = self.search("caffeine", frmt="cmlreact")
        r2 = self.search("caffeine", frmt="biopax2")
        try:
            self.search("caffeine", frmt="biopaxddddddd2")
            assert False
        except:
            assert True

    def test_entry_cmlreact(self):
        self.entry(10280, "cmlreact")

    def test_entry_biopax2(self):
        self.entry(10280, "biopax2")

    def test_entry_rxn(self):
        self.entry(10090, "rxn")

    def test_entry_badformat(self):
        try:
            self.entry(10090, "ggg")
            assert False
        except:
            assert True








from bioservices.wsdbfetch import WSDbfetch
from nose.plugins.attrib import attr


@attr('skip_travis')
class test_WSDbfetch(object):
    @classmethod
    def setup_class(klass):
        klass.s = WSDbfetch(verbose=False)

    def test_getSupportedDBs(self):
        res = self.s.getSupportedDBs()
        res = self.s.getSupportedDBs()
        assert len(res) >10

    @attr('slow')
    def test_getSupportedFormats(self):
        res = self.s.getSupportedFormats()
        res = self.s.getSupportedFormats()
        assert len(res) >10

    @attr('slow')
    def test_getSupportedStyles(self):
        res = self.s.getSupportedStyles()
        res = self.s.getSupportedStyles()
        assert len(res) >10

    @attr('fixme')
    def test_fetchBatch(self):
        self.s.fetchBatch("uniprot" ,"wap_mouse", "xml") 

    def test_fetchData(self):
        self.s.fetchData('uniprot:zap70_human')

    def test_getDatabaseInfo(self):
        res = self.s.getDatabaseInfo("uniprotkb")
        assert res.displayName == 'UniProtKB'

    def test_getDatabaseInfoList(self):
        assert len(self.s.getDatabaseInfoList())>10

    def test_getDatavaseInfoList(self):
        self.s.getDatabaseInfoList()

    def test_getDbFormats(self):
        self.s.getDbFormats("uniprotkb")

    def test_getFormat(self):
        assert len(self.s.getFormatStyles("uniprotkb", "fasta")) >= 3
        #['default', 'raw', 'html']

    def test_wrong_db(self):
        try:
            self.s.getDbFormats("uniprot")
            assert False
        except:
            assert True
        



from bioservices import readseq
from nose.plugins.attrib import attr



fasta = """MDAPRQVVNFGPGPAKLPHSVLLEIQKELLDYKGVGISVLEMSHRSSDFAKIINNTENLVRELLAVPDNYKVIFLQGGGCGQFSAVPLNLIGLKAGRCADYVVTGAWSAKAAEEAKKFGTINIVHPKLGSYTKIPDPSTWNLNPDASYVYYCANETVHGVEFDFIPDVKGAVLVCDMSSNFLSKPVDVSKFGVIFAGAQKNVGSAGVTVVIVRDDLLGFALRECPSVLEYKVQAGNSSLYNTPPCFSIYVMGLVLEWIKNNGGAAAMEKLSSIKSQTIYEIIDNSQGFYVCPVEPQNRSKMNIPFRIGNAKGDDALEKRFLDKALELNMLSLKGHRSVGGIRASLYNAVTIEDVQKLAAFMKKFLEMHQL"""


@attr('fixme') # does not work on travis
def test_readseq():

    # 8 is 
    # 2 is GenBank
    s = readseq.Readseq()
    jobid = s.run("cokelaer@test.co.uk", "test", sequence=fasta, inputformat=8, 
            outputformat=2)
    # should fail
    genbank = s.get_result(s._jobid)

    import time
    count = 0
    while s.get_status(s._jobid) != "FINISHED" and count<20:
        count += 1
        time.sleep(1)
        print(count)

    genbank = s.get_result(s._jobid)


    # now let us do the inverse
    jobid = s.run("cokelaer@ebi.co.uk", "test", sequence=genbank, inputformat=2, 
            outputformat=8)
    #FIXME do not know yet how to check



from bioservices.chembl import ChEMBL
from nose.plugins.attrib import attr

class test_Chembl(ChEMBL):

    def __init__(self):
        super(test_Chembl, self).__init__(verbose=False, 
                cache=False)
        self.default_extension = "xml"
        try:
            self.default_extension = "xmlf"
            assert False
        except:
            assert True

    def test_status(self):
        self.status()

    # COMPOUNDS RELATED ---------------------------------------------------------------

    def test_get_compounds_by_chemblId_form(self):
        res = self.get_compounds_by_chemblId_form("CHEMBL3")
        res[0]['parent']
        res = self.get_compounds_by_chemblId_form(["CHEMBL3"])
        assert len(res)

    def test_get_compounds_by_chemblId_drug_mechanism(self):
        res = self.get_compounds_by_chemblId_drug_mechanism("CHEMBL3")

    def test_get_individual_compounds_by_inChiKey(self):
        res = self.get_individual_compounds_by_inChiKey(self._inChiKey_example)
        assert res['molecularFormula'] == 'C19H21ClFN3O3'

    def test_get_compounds_by_chemblId(self):
        assert self.get_compounds_by_chemblId("CHEMBL1")
        # FIXME
        #assert self.get_compounds_by_chemblId("WRONG") in self._error_codes
        self.get_compounds_by_chemblId(self._chemblId_example)
        assert len(self.get_compounds_by_chemblId(["CHEMBL1", "CHEMBL2"]))==2

    def test_get_compounds_activities(self):
        self.get_compounds_activities("CHEMBL1")

    def test_get_compounds_by_SMILES(self):
        res = self.get_compounds_by_SMILES(self._smiles_example)

    def test_get_compounds_containing_SMILES(self):
        #res = self.get_compounds_containing_SMILES(self._smiles_struct_example)
        res = self.get_compounds_substructure(self._smiles_struct_example)

    def test_get_compounds_similar_to_SMILES(self):
        # at the time of the test, the length was 9
        res = self.get_compounds_similar_to_SMILES(self._smiles_example, similarity=90)
        assert len(res)>=9

    # TARGETS -----------------------------------------------------------------

    def test_get_target_uniprotId(self):
        res = self.get_target_by_uniprotId(self._target_uniprotId_example)
        assert res['proteinAccession'] == self._target_uniprotId_example

    def test_get_target_bioactivities(self):
        res = self.get_target_bioactivities(self._target_bioactivities_example)
        assert len(res)>1000

    def test_get_target_by_chemblId(self):
        resjson = self.get_target_by_chemblId(self._target_chemblId_example)

    def test_get_all_targets(self):
        assert len(self.get_all_targets())>1000

    def test_get_target_by_refseq(self):
        self.get_target_by_refseq(self._target_refseq_example)

    # ASSAYS ----------------------------------------------------------

    def test_get_assays_bioactivities(self):
        res = self.get_assays_bioactivities(self._assays_example)

    def test_get_assays(self):
        self.get_assays_bioactivities("CHEMBL1217643")
        self.get_assays_by_chemblId("CHEMBL1217643")

    @attr('skip')
    def test_inspect(self): 
        self.inspect(self._assays_example,'assay')

    def test_image(self):
        res = self.get_image_of_compounds_by_chemblId(
            self._image_chemblId_example
            , view=False)
        import os
        os.remove(self._image_chemblId_example + ".png")

    def test_version(self):
        self.version()

    def _test(self):

        """


    def get_target_by_refseq(self, query):

def get_assays_by_chemblId(self, query, frmt="json"):
    def get_assays_bioactivities(self, query, frmt="json"):
    def inspect(self, query, item_type):

    """



from bioservices import PDB


class test_PDB(PDB):
                                               
    @classmethod
    def setup_class(klass):
        klass.s = PDB(verbose=False)



    def test_pdb(self):
        res = self.s.get_file("1FBV", "pdb")
        res = self.s.get_file("1FBV", "xml", compression=True)
        res = self.s.get_file("1FBV", "cif")
    
        ids = self.s.get_current_ids()

    def test_ligands(self):
        '4HHB' in self.s.get_ligands("4HHB")

    def get_xml_query(self):
        query = """<?xml version="1.0" encoding="UTF-8"?>
<orgPdbQuery>
<version>B0907</version>
<queryType>org.pdb.query.simple.ExpTypeQuery</queryType>
<description>Experimental Method Search : Experimental Method=SOLID-STATE NMR</description>
<mvStructure.expMethod.value>SOLID-STATE NMR</mvStructure.expMethod.value>
</orgPdbQuery>
"""
        res = self.s.get_xml_query(query)
        assert len(res.split("\n"))>10






from bioservices import HGNC


def test_hgnc():

    h = HGNC()
    h.get_info()

    h.fetch('symbol', 'ZNF3')



    h.fetch('alias_name', 'A-kinase anchor protein, 350kDa')


    h.search('BRAF')
    h.search('symbol', 'ZNF*')
    h.search('symbol', 'ZNF?')
    h.search('symbol', 'ZNF*+AND+status:Approved')
    h.search('symbol', 'ZNF3+OR+ZNF12')
    h.search('symbol', 'ZNF*+NOT+status:Approved')




from bioservices import BioCarta


def test_biocarta():

    b = BioCarta()
    res = b.get_pathway_protein_names('h_RELAPathway')
    assert len(res) == 13

    b.organism = 'Homo sapiens'
    b.all_pathways



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id$
"""Interface to PubChem web service

.. topic:: What is PubChem ?

    :URL: http://pubchem.ncbi.nlm.nih.gov/pug_rest/

    .. highlights::

        TODO

        -- puchem web site, Oct 2014


"""
from bioservices.services import REST


__all__ = ["PubChem"]


class PubChem(REST):
    """Interface to the `PubChem <todo>`_ service

    """
    _url = "http://pubchem.ncbi.nlm.nih.gov/rest/pug"

    def __init__(self, verbose=False, cache=False):
        """**Constructor**

        :param verbose: set to False to prevent informative messages
        """
        print("PubChem is not finalised yet. This is currently only a draft version")
        super(PubChem, self).__init__(name="PubChem", url=PubChem._url,
                verbose=verbose, cache=cache)

    def get_compound_by_smiles(self, identifier, frmt='json'):

        res = self.http_get("compound/smiles/" + identifier 
                + '/cids/%s' % frmt, frmt=frmt,
                headers=self.get_headers(content=frmt))
        return res




#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
# $Id$
"""This module provides a class :class:`~BioDBNet` to access to BioDBNet WS.


.. topic:: What is BioDBNet ?

    :URL: http://biodbnet.abcc.ncifcrf.gov/
    :Service: http://biodbnet.abcc.ncifcrf.gov/webServices
    :Citations:  Mudunuri,U., Che,A., Yi,M. and Stephens,R.M. (2009) bioDBnet: the biological database network. Bioinformatics, 25, 555-556

    .. highlights::

        "BioDBNet Database is a repository hosting computational models of biological
        systems. A large number of the provided models are published in the
        peer-reviewed literature and manually curated. This resource allows biologists
        to store, search and retrieve mathematical models. In addition, those models can
        be used to generate sub-models, can be simulated online, and can be converted
        between different representational formats. "

        -- From BioDBNet website, Dec. 2012

    .. versionadded:: 1.2.3
    .. sectionauthor:: Thomas Cokelaer, Feb 2014

"""
import io
from bioservices.services import WSDLService

__all__ = ["BioDBNet"]


class BioDBNet(WSDLService):
    """Interface to the `BioDBNet <http://biodbnet.abcc.ncifcrf.gov/>`_ service


    ::

        >>> from bioservices import *
        >>> s = BioDBNet()

    Most of the BioDBNet WSDL are available. There are functions added to
    the original interface such as :meth:`extra_getReactomeIds`.

    Use :meth:`db2db` to convert from 1 database to some databases.
    Use :meth:`dbReport` to get the convertion from one database to all
    databases.

    """
    _url = 'http://biodbnet.abcc.ncifcrf.gov/webServices/bioDBnet.wsdl'
    def __init__(self, verbose=True, cache=False):
        """.. rubric:: Constructor

        :param bool verbose:

        """
        super(BioDBNet, self).__init__(name="BioDBNet", 
                url=BioDBNet._url, verbose=verbose, cache=cache)

    def _interpret_input_db(self, inputValues):
        if isinstance(inputValues, list):
            inputValues = ",".join(inputValues)
            return inputValues
        elif isinstance(inputValues, str):
            return inputValues
        else:
            raise NotImplementedError

    def _interpret_output_db(self, input_db, output_db):
        if isinstance(output_db, list):
            outputs = ",".join(output_db)
        else:
            outputs = output_db
        inputResult = self.getInputs()
        #getOutputsForInput method
        outputResult = self.getOutputsForInput(input_db)
        for output in outputs.split(","):
            if output not in outputResult:
                print(output + " not found")
                print(outputResult)
                raise Exception
        return outputs

    def db2db(self, input_db, output_db, inputValues, taxon=9606):
        """Retrieves models associated to the provided Taxonomy text.

        :param str text: free (Taxonomy based) text
        :return:  list of model identifiers

        ::

            >>> from bioservices import BioDBNet
            >>> input_db = 'Ensembl Gene ID'
            >>> output_db = ['Gene Symbol']
            >>> input_values = ['ENSG00000121410, ENSG00000171428']
            >>> print(s.db2db(input_db, output_db, input_values, 9606)
            Ensembl Gene ID Gene Symbol
            ENSG00000121410 A1BG
            ENSG00000171428 NAT1

        """
        inputValues = self._interpret_input_db(inputValues)
        outputs = self._interpret_output_db(input_db, output_db)

        # dbPath = 'Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID'
        # getDirectOutputsForInput method
        # directOutputResult = self.getDirectOutputsForInput(input_db)

        params = self.suds.factory.create("db2dbParams")
        params.taxonId = str(taxon)
        params.inputValues = inputValues
        params.input = input_db
        params.outputs = output_db

        res = self.serv.db2db(params)
        return res

    def dbFind(self, input_db, inputValues, taxon="9606"):
        inputValues = self._interpret_input_db(inputValues)

        params = self.suds.factory.create('dbFindParams')
        params.output = input_db
        params.taxonId = str(taxon) # do we need the cast ?
        params.inputValues = inputValues
        return self.serv.dbFind(params)


    def dbOrtho(self, input_db, output_db, inputValues, input_taxon, output_taxon):
        inputValues = self._interpret_input_db(inputValues)
        outputs = self._interpret_output_db(input_db, output_db)

        params = self.suds.factory.create("dbOrtho")
        params.input = input_db
        params.output = output_db
        params.inputTaxon = str(input_taxon)
        params.outputTaxon = str(output_taxon)
        return self.serv.dbOrtho(params)

    def dbReport(self, input_db, inputValues, taxon=9606, output="raw"):
        """Returns report

        :param output: returns dataframe if set to dataframe

        ::

            s.dbReport("Ensembl Gene ID", ['ENSG00000121410', 'ENSG00000171428'])


        """
        inputValues = self._interpret_input_db(inputValues)

        params = self.suds.factory.create('dbReportParams')
        params.input = input_db
        params.taxonId = str(taxon) # do we need the cast ?
        params.inputValues = inputValues

        res = self.serv.dbReport(params)
        if output == "dataframe":
            try:
                import pandas as pd
            except:
                print("Pandas library is not installed. dataframe are not  available")
            df = pd.readcsv(io.StringIO(res.strip()), sep="\t")
            return df
        else:
            return res

    def dbWalk(self, dbPath, inputValues, taxon=9606):
        dbPath = 'Ensembl Gene ID->Gene ID->Homolog - Mouse Gene ID->Ensembl Gene ID'
        inputValues = self._interpret_input_db(inputValues)
        params = self.suds.factory.create("dbWalkParams")
        params.dbPath = dbPath
        params.inputValues = inputValues
        params.taxonId = str(taxon)
        return self.serv.dbWalk(params)

    def getDirectOutputsForInput(self, input_db):
        return self.serv.getDirectOutputsForInput(input_db).split(",")

    def getInputs(self):
        """Return list of possible input database

        ::

            s.getInputs()
        """
        return self.serv.getInputs().split(",")

    def getOutputsForInput(self, input_db):
        """Return list of possible output database for a given input database

        ::

            s.getOutputsForInput("UniProt Accession")

        """
        if input_db not in self.getInputs():
            raise ValueError("Invalid input database provided")
        return self.serv.getOutputsForInput(input_db).split(",")






from bioservices import Ensembl


class test_Ensembl(Ensembl):
                                           
    @classmethod
    def setup_class(klass):
        klass.s = Ensembl(verbose=False)

    def test_get_archive(self):
        res = self.s.get_archive("ENSG00000157764")
        assert 'id' in res.keys()

    def test_post_archive(self):
        res = self.s.post_archive(["ENSG00000157764"])
        assert "ENSG00000157764" in [x['id'] for x in res]
       
    def test_genetree_by_id(self):
        res = self.s.get_genetree_by_id("ENSGT00390000003602", 
                frmt='phyloxml', aligned=False, sequence='cdna')[0:1000]
        assert "taxonomy" in res
        assert "117571" in res
        assert "Euteleos" in res

        self.s.get_genetree_by_id('ENSGT00390000003602', frmt='nh', 
                nh_format='simple')
        self.s.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml')
        self.s.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml',
                aligned=True, sequence='cdna')
        self.s.get_genetree_by_id('ENSGT00390000003602', frmt='phyloxml', 
                sequence='none')

    def test_get_genetree_by_member_id(self):
        res = self.s.get_genetree_by_member_id('ENSG00000157764', 
                frmt='json', nh_format='phylip')
        assert len(res)>1000

    def test_get_genetree_by_member_symbol(self):
        res = self.s.get_genetree_by_member_symbol('human', 'BRCA2', 
                                                      nh_format='simple')
        # was working until dec 2014 then results changed. so not stable
        #assert res[0:200] == """((((((((ENSPFOP00000001575:0.046083,ENSXMAP00000006983:0.065551):0.43822,ENSONIP00000006940:0.359035):0.019582,((ENSTRUP00000015030:0.077336,ENSTNIP00000002435:0.099898):0.208834,ENSGACP00000015199:0."""

    def test_get_alignment_by_region(self):
        region = '2:106040000-106040050'
        species = 'taeniopygia_guttata'
        res = self.s.get_alignment_by_region(region, species, 
                                                species_set_group='sauropsids')
        
        assert 'gallus_gallus' in res[0]['tree']



    def test_get_homology_by_id(self):
        res = self.s.get_homology_by_id('ENSG00000157764')
        assert res.keys()
        res = self.s.get_homology_by_id('ENSG00000157764', frmt='xml')
        res = self.s.get_homology_by_id('ENSG00000157764', format='condensed', 
                                  type='orthologues', target_taxon='10090')
        assert 'homologies' in res['data'][0].keys()

    def test_references(self):
        res = self.s.get_xrefs_by_id('ENST00000288602', external_db='PDB', 
                                                    all_levels=True)
        assert res[0]['dbname'] == 'PDB'

        res = self.s.get_xrefs_by_name('BRCA2', 'human')    
        assert 'db_display_name' in res[0].keys()

        res = self.s.get_xrefs_by_symbol('BRCA2', 'homo_sapiens',
                                     external_db='HGNC')    
        assert 'id' in res[0]

    def test_info(self):
        assert len(self.s.get_info_analysis('human')) > 0
        assert len(self.s.get_info_assembly('human')['karyotype']) == 25

        self.s.get_info_assembly_by_region('homo_sapiens', 'X')

        assert len(self.s.get_info_biotypes('human'))

        assert self.s.get_info_compara_methods()

        assert self.s.get_info_compara_by_method('EPO')[0]['method'] == 'EPO'

        assert self.s.get_info_comparas()['comparas']

        assert self.s.get_info_data()

        res = self.s.get_info_external_dbs('human')
        assert len([x['name'] for x in res if 'hgnc' in x['name'].lower()])>=5

        assert self.s.get_info_ping()
        assert self.s.get_info_rest()
        assert self.s.get_info_software()
        res = self.s.get_info_species()
        assert 'ovis_aries' in [x['name'] for x in res['species'] if 'ovis' in x['name']]

    def test_lookup(self):
        res = self.s.get_lookup_by_id('ENSG00000157764', expand=True)
        
        res = self.s.get_lookup_by_id('ENSG00000157764', expand=True)
        assert res.keys()
        
        res = self.s.post_lookup_by_id(["ENSG00000157764", "ENSG00000248378" ], 
                    expand=0)

        res = self.s.get_lookup_by_symbol('homo_sapiens', 'BRCA2', expand=True)
        assert len(res['Transcript'])
   
        res = self.s.post_lookup_by_symbol('human', ["BRCA2", "BRAF" ], expand=True)
        assert len(res['BRCA2']['Transcript'])


    def test_mapping(self):
        res = self.s.get_map_assembly_one_to_two('GRCh37', 'GRCh38', 
                  region='X:1000000..1000100:1')
            
        res = self.s.get_map_translation_to_region('ENSP00000288602', '100..300')
        assert res['mappings'][0]  

        res = self.s.get_map_cds_to_region('ENST00000288602', '1..1000')

        res = self.s.get_map_cdna_to_region('ENST00000288602', '100..300')

    def test_ontologies(self):
        res = self.s.get_ontology_ancestors_by_id('GO:0005667')
        res = self.s.get_ontology_ancestors_chart_by_id('GO:0005667')
        res = self.s.get_ontology_descendants_by_id('GO:0005667')
        res = self.s.get_ontology_by_id('GO:0005667')
        assert res['accession']

        res = self.s.get_ontology_by_name('transcription factor complex')
        res = self.s.get_taxonomy_classification_by_id(9606)
        assert res[0]['children']
        res = self.s.get_ontology_by_id('GO:0005667')
        assert 'children' in res.keys()
        res = self.s.get_ontology_by_name('transcription factor complex')
        assert res[0]['children']
        res = self.s.get_ontology_by_name('transcription factor')
        assert res == 400 # error here


    def test_taxonomy(self):

        res = self.s.get_taxonomy_by_name('homo')
        assert len(res[0])>1

        res = self.s.get_taxonomy_classification_by_id('9606')

        res = self.s.get_taxonomy_by_name('Homo')

        self.s.get_taxonomy_by_id(9606)['scientific_name']

        res = self.s.get_overlap_by_id("ENSG00000157764", feature='gene')

        res = self.s.get_overlap_by_region('7:140424943-140624564', 
            species='human', feature='gene')

        res = self.s.get_overlap_by_translation('ENSP00000288602', 
                type='Superfamily')

        res = self.s.get_overlap_by_translation('ENSP00000288602', 
            type='missense_variant',
            feature='transcript_variation')

        res = self.s.get_overlap_by_translation('ENSP00000288602', 
            type='missense_variant',
            feature='somatic_transcript_variation')

    def test_regulation(self):
        res = self.s.get_regulatory_by_id('ENSR00001348195', 'human')
        assert 'ID' in res[0].keys()
   
    def test_sequences(self):
        sequence = self.s.get_sequence_by_id('ENSG00000157764', frmt='text')
        assert sequence[0:120] == """CGCCTCCCTTCCCCCTCCCCGCCCGACAGCGGCCGCTCGGGCCCCGGCTCTCGGTTATAAGATGGCGGCGCTGAGCGGTGGCGGTGGTGGCGGCGCGGAGCCGGGCCAGGCTCTGTTCAA"""

        sequence = self.s.get_sequence_by_id('ENSG00000157764', frmt='fasta')
        assert sequence.startswith(">ENSG00000157764 chromosome:")

        sequence = self.s.get_sequence_by_id('CCDS5863.1', frmt='fasta', 
            object_type='transcript', db_type='otherfeatures',
            type='cds', species='human')

        sequence = self.s.get_sequence_by_id('ENSG00000157764', frmt='seqxml',
            multiple_sequences=True,type='protein')
        sequence = self.s.get_sequence_by_region('X:1000000..1000100:1', 'human')
        assert sequence['id'] == 'chromosome:GRCh38:X:1000000:1000100:1'
        sequence = self.s.get_sequence_by_region('ABBA01004489.1:1..100', 'human',
                frmt='json', coord_system='seqlevel')

        seq = self.s.get_sequence_by_id('ENSE00001154485', expand_5prime=10,
                frmt='fasta', type='genomic')
        assert seq == u'>ENSE00001154485 chromosome:GRCh38:7:140924566:140924774:-1\nCCCAGCTCTCCGCCTCCCTTCCCCCTCCCCGCCCGACAGCGGCCGCTCGGGCCCCGGCTC\nTCGGTTATAAGATGGCGGCGCTGAGCGGTGGCGGTGGTGGCGGCGCGGAGCCGGGCCAGG\nCTCTGTTCAACGGGGACATGGAGCCCGAGGCCGGCGCCGGCGCCGGCGCCGCGGCCTCTT\nCGGCTGCGGACCCTGCCATTCCGGAGGAG\n'

        res = self.s.get_sequence_by_id('ENSG00000157764', frmt='seqxml', 
                type='protein', multiple_sequences=1)
        assert 'XSTT' in res 

        res = self.s.get_sequence_by_id('GENSCAN00000000001', frmt='json', 
                object_type='predictiontranscript', db_type='core', 
                species='homo_sapiens', type='protein')
        assert 'MERGKK' in res['seq']

        res = self.s.get_sequence_by_id('ENSP00000288602', frmt='json')
        assert res['seq'].startswith("MAAL")
    
    def test_variation(self):
        res = self.s.get_variation_by_id('rs56116432', 'human')
        assert 'MAF' in res.keys()
        res = self.s.get_vep_by_id('COSM476', 'human')
        res = self.s.get_vep_by_id('rs116035550', 'human')
        res = self.s.get_vep_by_region('9:22125503-22125502:1', 'C', 'human')
        assert res[0]['most_severe_consequence']

        self.s.get_variation_by_id("rs56116432", species='homo_sapiens')
        self.s.get_variation_by_id("rs56116432", species='homo_sapiens', 
                pops=1, genotypes=1, phenotypes=1)




a = test_Ensembl()
a.setup_class()
a.test_get_alignment_by_region()



from bioservices.apps.fasta import FASTA, MultiFASTA
import tempfile
from nose.plugins.attrib import attr


class test_FASTA(object):
    @classmethod
    def setup_class(klass):
        klass.s = FASTA(verbose=False)

    def test_fasta(self):
        self.s.load_fasta(None)
        self.s.load_fasta("P43403")
        self.s.load_fasta("P43403") # already there 
        self.s.header
        self.s.gene_name
        self.s.sequence
        self.s.fasta
        self.s.identifier
        fh = tempfile.NamedTemporaryFile(delete=False)
        self.s.save_fasta(fh.name)
        self.s.read_fasta(fh.name)
        fh.delete = True
        fh.close()

class test_FASTA(object):
    @classmethod
    def setup_class(klass):
        klass.s = MultiFASTA()

        try:
            klass.s.fasta
            assert False
        except:
            assert True
        try:
            f.header
            assert False
        except:
            assert True

        klass.s.load_fasta("P43403")
        klass.s.load_fasta("P43408")

    def test_attributes(self):
        assert len(self.s) == 2
    
        self.s.ids
        self.s.fasta
        fh = tempfile.NamedTemporaryFile(delete=False)
        self.s.save_fasta(fh.name)
        self.s.read_fasta(fh.name)

        self.s.fasta["P43403"]

    @attr('skip')
    def test_extra(self):

        self.s.hist_size()
        self.s.df



from bioservices.uniprot import UniProt
from nose.plugins.attrib import attr


class test_UniProt(UniProt):
    def __init__(self):
        super(test_UniProt, self).__init__(verbose=False, cache=False)
        self.debugLevel = "ERROR"

    def test_mapping(self):
        res = self.mapping(fr="ACC+ID", to="KEGG_ID", query='P43403')
        assert res['P43403'] == ['hsa:7535']
        try:
            res = self.mapping(fr="AC", to="KEID", query='P434')
            assert False
        except:
            assert True

    @attr('slow')
    def test_retrieve(self):
        self.retrieve("P09958", frmt="rdf")
        self.retrieve("P09958", frmt="xml")
        self.retrieve("P09958", frmt="txt")
        self.retrieve("P09958", frmt="fasta")
        self.retrieve("P09958", frmt="gff")

        try:
            self.retrieve("P09958", frmt="dummy")
            assert False
        except:
            assert True

    @attr('slow')
    def test_search(self):
        self.search('zap70+AND+organism:9606', frmt='list')
        self.search("zap70+and+taxonomy:9606", frmt="tab", limit=3,
                    columns="entry name,length,id, genes, genes(PREFERRED), interpro, interactor")
        self.search("zap70+and+taxonomy:9606", frmt="tab", limit=3,
                    columns="entry name, go(biological process), comment(FUNCTION), comment(DOMAIN)")

        self.search("ZAP70_HUMAN", frmt="tab", columns="sequence", limit=1)

        self.quick_search("ZAP70")

    @attr('skip')
    def test_uniref(self):
        df = self.uniref("member:Q03063")
        df.Size

    @attr('skip')
    def test_get_df(self):
        df = self.get_df(["P43403"])

    def test_fasta(self):
        "Q9Y617" in self.get_fasta(["Q9Y617-1"])
        "Q9Y617" not in self.get_fasta_sequence(["Q9Y617-1"])



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2013-2014 - EBI-EMBL
#
#  File author(s):
#      Patrick Short
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://github.com/cokelaer/bioservices
#  documentation: http://packages.python.org/bioservices
#
##############################################################################

# About Clinvitae (http://clinvitae.invitae.com/)

# "CLINVITAE is a database of clinically-observed genetic variants aggregated
# from public sources, operated and made freely available by INVITAE." - CLINVITAE website

from bioservices.services import REST

__all__ = ["Clinvitae"]


class Clinvitae(REST):
    """
    class for interfacing with the Clinvitae web service

    Requests will return a list of json dicts. each dict has the following fields::

        accessionId
        gene
        nucleotideChanges
        description
        classification
        reportedClassification
        url
        region
        proteinChange
        lastUpdated
        alias
        source
        acmgClassification
        submitter
        defaultNucleotideChange
        _id
        transcripts
        lastEvaluated

    example query on BRCA1::

        >>> c = Clinvitae()
        >>> res = c.query_gene('brca1')
        >>> entry1 = res[0]
        >>> print(entry1.keys())  # display fields for first entry
        >>> print(entry1['accessionId']) # accession id for first entry
        >>> print(entry1['lastEvaluated'])  # date first variant entry was last evaluated
        >>> print(entry1['source'])  # source of first variant entry

    """

    def __init__(self):
        url = "http://clinvitae.invitae.com/api/v1"
        super(Clinvitae, self).__init__("Clinvitae", url=url)

    def query_gene(self, gene):
        """
        takes gene name and returns json of variants in gene (not case sensitive)

        ::

            >>> c = Clinvitae()
            >>> res = c.query_gene('brca1')
            >>> entry1 = res[0]
            >>> print(entry1['accessionId'])  # accession id for first entry
            u'SCV000039520'
            >>> print(entry1['lastEvaluated'])  # date first variant entry was last evaluated
            u'2013-04-03'
            >>> print(entry1['source']0  # source of first variant entry
            u'ClinVar'

        """
        return self.http_get("variants?q=%s" % gene, frmt='json')

    def query_hgvs(self, hgvs):
        """Takes an hgvs (variant) tag and returns ALL reported variants in the gene in which hgvs is located

        ::

            >>> c = Clinvitae()
            >>> res = c.query_gene('NM_198578.3:c.1847A>G')  # returns all entries in LRRK2 gene
            >>> entry1 = res[0]
            >>> print(entry1['accessionId']) # accession id for first entry
            u'SCV000056058'
            >>> print(entry1['lastEvaluated']) # date first variant entry was last evaluated
            u'2012-09-13'
            >>> print(entry1['source']) # source of first variant entry
            u'ClinVar'

        """
        return self.http_get("variants?q=%s" % hgvs, frmt='json')

    def all_variants(self, gene):
        """
        returns a list of unique hgvs tags reported in gene

        ::

            >>> c = Clinvitae()
            >>> res = c.all_variants('MUTYH')  # returns all reported variants in MUTYH gene
            >>> print(res[0:5])
            [u'NM_001048171.1:c.-2188C>T',
            u'NM_001048171.1:c.462+35A>G',
            u'NM_001048171.1:c.1099G>T',
            u'NM_001048171.1:c.972G>C',
            u'NM_001048171.1:c.1476+35C>T']

        """
        results = self.query_gene(gene)
        variants = set()
        for result in results:  # each result is json dict of single entry
            variants.add(result['defaultNucleotideChange'])
        return list(variants)

    def get_pathogenic(self, gene):
        """
        returns all fields for entries reported as Pathogenic by any source in Clinvitae

        ::

            >>> c = Clinvitae()
            >>> pathogenic = c.get_pathogenic('brca1')  # returns pathogenic or likely pathogenic
            >>> len(pathogenic)  # number of pathogenic variants
            1100

        """
        results = self.query_gene(gene)
        variants = set()
        for result in results:
            if result['reportedClassification'].lower() in ['pathogenic', 'likely pathogenic']:
                variants.add(result['defaultNucleotideChange'])
        return list(variants)

    def get_benign(self, gene):
        """
        returns all fields for entries reported as Benign by any source in Clinvitae

        ::

            >>> c = Clinvitae()
            >>> benign = c.get_benign('brca1')  # returns benign or likely benign
            >>> len(benign)  # number of benign variants
            187

        """
        results = self.query_gene(gene)
        variants = set()
        for result in results:
            if result['reportedClassification'].lower() in ['benign', 'likely benign']:
                variants.add(result['defaultNucleotideChange'])
        return list(variants)

    def get_VUS(self, gene):
        """
        returns all fields for entries not classified as benign or pathogenic -> variant of unknown significance (VUS)

        ::

            >>> c = Clinvitae()
            >>> vus = c.get_VUS('brca1')
            >>> len(vus)  # number of benign variants
            2389
        """
        results = self.query_gene(gene)
        variants = set()
        for result in results:
            if result['reportedClassification'].lower() not in ['pathogenic', 'likely pathogenic', 'benign', 'likely benign']:
                variants.add(result['defaultNucleotideChange'])
        return list(variants)



from bioservices import PRIDE

p = PRIDE()


def test_pride_project():
    # p = PRIDE()
    res = p.get_project("PRD000001")
    assert res["numPeptides"] == 6758

    projects = p.get_project_list(show=100)
    counter = p.get_project_count()
    assert counter > 1000


def test_pride_assay():
    # p = PRIDE()
    res = p.get_assays(1643)
    assert res['proteinCount'] == 276

    assays = p.get_assay_list('PRD000001')
    count = p.get_assay_count('PRD000001')

    assert count == len(assays)


def test_pride_file():
    # p = PRIDE()
    files = p.get_file_list("PRD000001")
    count = p.get_file_count("PRD000001")
    assert len(files) == count

    files = p.get_file_list_assay(1643)
    count = p.get_file_count_assay(1643)
    assert len(files) == count


def test_pride_protein():
    files = p.get_protein_list("PRD000001")
    count = p.get_protein_count("PRD000001")
    assert count == 1530
    assert len(files) == 10  # only a subset

    files = p.get_protein_list_assay(1643)
    count = p.get_protein_count_assay(1643)
    # assert len(files) == count
    assert count == 276
    assert len(files) == 10


def test_pride_peptide():
    # without sequece
    count = p.get_peptide_count('PRD000001')
    peptides = p.get_peptide_list('PRD000001', show=count)
    assert len(peptides) == count

    # with sequence
    peptides = p.get_peptide_list('PRD000001', sequence='PLIPIVVEQTGR')
    assert len(peptides) == 4
    count = p.get_peptide_count('PRD000001', sequence='PLIPIVVEQTGR')
    assert count == 4

    # with assay and sequence
    peptides = p.get_peptide_list_assay(1643, sequence='AAATQKKVER')
    assert len(peptides) == 5
    count = p.get_peptide_count_assay(1643, sequence='AAATQKKVER')
    assert count == 5

    # with assay no sequence
    count = p.get_peptide_count_assay(1643)
    assert count == 1696
    peptides = p.get_peptide_list_assay(1643)
    assert len(peptides) == 10
